{
  HTML Help support for Delphi 6 VCL and CLX mode
  Version 1.0
  Author: Antti Lukats antti@case2000.com
  http://www.case2000.com

  just place this unit into 'uses' clause!

}
unit C2HtmlHelp;

interface

// undefine for VCL mode!
{$DEFINE CLX}

uses
  Types, WinTypes,

{$IFDEF CLX}
  QForms, QDialogs, Qt,
{$ELSE}
  Forms,
{$ENDIF}
  HtmlHelpAPI,
  HelpIntfs,
  WinHelpViewer,
  SysUtils, Classes;

type

 TC2HelpViewer = class(TInterfacedObject, ICustomHelpViewer, IExtendedHelpViewer, IWinHelpTester)
  private
   FViewerID: Integer;
   FLastCommand: String;
   FLastContext: Integer;

   FHelpFile: String;
   HelpFile: Array[0..259] of char;
   function HelpHandle: hWnd;
  public
   FHelpManager: IHelpManager;
   constructor Create;
   destructor Destroy; override;

   procedure InternalShutDown; // do we need?

   { ICustomHelpViewer }
   function GetViewerName : String;
   function UnderstandsKeyword(const HelpString: String): Integer;
   function GetHelpStrings(const HelpString: String): TStringList;
   function CanShowTableOfContents: Boolean;
   procedure ShowTableOfContents;
   procedure ShowHelp(const HelpString: String);
   procedure NotifyID(const ViewerID: Integer);
   procedure SoftShutDown;
   procedure ShutDown;

   { IExtendedHelpViewer }
   function UnderstandsTopic(const Topic: String): Boolean;
   procedure DisplayTopic(const Topic: String);
   function UnderstandsContext(const ContextID: Integer; const HelpFileName: String): Boolean;
   procedure DisplayHelpByContext(const ContextID: Integer;  const HelpFileName: String);

   { IWinHelpTester }
    function CanShowALink(const ALink, FileName: String): Boolean;
    function CanShowTopic(const Topic, FileName: String): Boolean;
    function CanShowContext(const Context: Integer; const FileName: String): Boolean;

    function GetHelpPath : String;
    function GetDefaultHelpFile: String;
  end;

implementation

var
  C2HelpViewer : TC2HelpViewer;

{ TC2HelpViewer }

function TC2HelpViewer.CanShowALink(const ALink, FileName: String): Boolean;
begin
  result := False;
end;

function TC2HelpViewer.CanShowContext(const Context: Integer; const FileName: String): Boolean;
begin
  result := False;
end;

function TC2HelpViewer.CanShowTableOfContents: Boolean;
begin
  result := False;
end;

function TC2HelpViewer.CanShowTopic(const Topic, FileName: String): Boolean;
begin
  result := False;
end;

constructor TC2HelpViewer.Create;
begin

  inherited Create;
end;

destructor TC2HelpViewer.Destroy;
begin

  inherited Destroy;
end;

procedure TC2HelpViewer.DisplayHelpByContext(const ContextID: Integer; const HelpFileName: String);
var
  a: Array[0..259] of char;
begin
  StrPCopy(a, FHelpFile);

  HtmlHelpA(HelpHandle, a, HH_HELP_CONTEXT, ContextID);  //display help context
end;

procedure TC2HelpViewer.DisplayTopic(const Topic: String);
var
  a: Array[0..259] of char;
begin
  StrPCopy(a, FHelpFile+ '::/'+Topic+'.htm');
  HtmlHelpA(HelpHandle, a, HH_DISPLAY_TOPIC, 0);
end;

function TC2HelpViewer.GetDefaultHelpFile: String;
begin
          { TODO : Maybe needs change }
  FHelpFile := Application.HelpFile;
  result := FHelpFile;
end;

function TC2HelpViewer.GetHelpPath: String;
begin
  //
end;

function TC2HelpViewer.GetHelpStrings(const HelpString: String): TStringList;
begin
      { TODO : We have to return something }
end;

function TC2HelpViewer.GetViewerName: String;
begin
  result := 'HtmlHelp';
end;

function TC2HelpViewer.HelpHandle: hWnd;
begin
{$IFDEF CLX}
  result := QWidget_winid(Application.MainForm.Handle);
{$ELSE}
  result := Application.MainForm.Handle;
{$ENDIF}
end;

procedure TC2HelpViewer.InternalShutDown;
begin
  SoftShutDown;
  if Assigned(FHelpManager) then
  begin
    FHelpManager.Release(FViewerID);
    if Assigned(FHelpManager) then FHelpManager := nil;
  end;
end;

procedure TC2HelpViewer.NotifyID(const ViewerID: Integer);
begin
  FViewerID := ViewerID;
end;

procedure TC2HelpViewer.ShowHelp(const HelpString: String);
var
  a: Array[0..259] of char;
  keyword: Array[0..32] of char;
begin
  // Help File
  StrPCopy(a, FHelpFile);

  // Keyword to Show
  StrPCopy(Keyword, HelpString);
  HtmlHelpA(HelpHandle, a, HH_DISPLAY_INDEX, Integer(@keyword));  //display keyword
end;

procedure TC2HelpViewer.ShowTableOfContents;
begin
         { TODO : Unsolved - PROBLEM }
// Never called as WinHelp will always get called
//  ShowMessage('Show TOC');
end;

procedure TC2HelpViewer.ShutDown;
begin
  if Assigned(FHelpManager) then FHelpManager := nil;
end;

procedure TC2HelpViewer.SoftShutDown;
begin
  //
end;

function TC2HelpViewer.UnderstandsContext(const ContextID: Integer; const HelpFileName: String): Boolean;
begin
  result := true;
end;

function TC2HelpViewer.UnderstandsKeyword(const HelpString: String): Integer;
begin
             { TODO : What is correct response? }
  result := 1;
end;

function TC2HelpViewer.UnderstandsTopic(const Topic: String): Boolean;
begin
  result := true;
end;


initialization
  C2HelpViewer := TC2HelpViewer.Create;
  HelpIntfs.RegisterViewer(C2HelpViewer, C2HelpViewer.FHelpManager);

  // Set WinHelpWiewer Hack!
  WinHelpTester := C2HelpViewer;

finalization

  if Assigned(C2HelpViewer.FHelpManager) then
  begin
    C2HelpViewer.InternalShutDown;
  end;

end.
