unit C_LinearMemory;
{** C_LinearMemory.pas
 **  JF - Mai 2000
 **
 **  This unit defines 2 object (TWriteLinearMemory and TReadLinearMemory) used
 ** to write /read data (integer, single, double, char, pchar, buffer, string...)
 ** into a linear memory buffer, which can be accessed with the Buf property
 **}

{$DEFINE GROWEABLE_MEMORY}
{** Dfinissez GROWEABLE_MEMORY ou NO_GROWEABLE_MEMORY pour
 ** permettre l'augmentation automatique du buffer (!! exprimental, il
 ** ne vaut mieux pas le dfinir
 **}
interface

uses
  Windows;
type
  TArrayOfByte = array[0..512] of char;
  pTArrayOfByte= ^TArrayOfByte;

  TWriteLinearMemory = class
  private
    FMemBuf : pTArrayOfByte; // Pointeur sur le dbut de la zone alloue
    FSize   : integer;       // taille de la zone alloue
    FIntPos : integer;       // Position
    FAutoGrow : boolean;     // Si l'utilisateur ne cr pas de buffer, il faut augmenter la taille du buffer automatiquement.
    procedure SetSize(s : integer);
  public
    constructor Create ;
    destructor  Destroy ; override;

    // Standard numeral writer integer
    procedure   WriteByte(b : byte);          // 0..255
    procedure   WriteChar(c : char);          // #0..#255
    procedure   WriteShortInt(si : ShortInt); // -128..127
    procedure   WriteWord(w : word);          // 0..65535
    procedure   WriteSmallInt(si : SmallInt); // -32768..32767
    procedure   WriteCardinal(c : Cardinal);  // 0..2^32-1
    procedure   WriteInteger(i : integer);    // -2^31..2^31+1
    // Standard numeral writer float
    procedure   WriteSingle(s : single);
    procedure   WriteDouble(d : double);      // use also for TDateTime, TTime and TDate
    // Complex structure
    procedure   WriteStringAZT(s : string);   // Write char until the first #0
    Procedure   WriteLongString(s: string);   // Write string, even if there is some #0 in it.
    // Generic buffer
    procedure   WriteBuf(buf : pointer ; len : integer);
    procedure   WriteRawString(s : string);
    // File functions
    procedure   SaveToFile(filename : string);
    procedure   AppendToFile(fd : THANDLE);
    property    Buf : pTArrayOfByte read FMemBuf write FMemBuf;
    property    Size : integer read FSize write SetSize;
    property    Pos : integer read FIntPos;
    Property    AutoGrow : boolean read FAutoGrow write FAutoGrow;
  end;

  TReadLinearMemory = class
  private
    FMemBuf : pTArrayOfByte; // Pointeur sur le dbut de la zone alloue
    FSize   : integer;       // taille de la zone alloue
    FIntPos : integer;       // indice dans la zone alloue
    procedure SetMemBuf(buf : pTArrayOfByte);
  public
    constructor Create ;
    destructor  Destroy ; override;
    function    ReadByte    : byte;
    function    ReadChar    : char;
    function    ReadShortInt: shortint;
    function    ReadWord    : word;
    function    ReadSmallInt: SmallInt;
    function    ReadCardinal: Cardinal;  
    function    ReadInteger : integer;
    function    ReadSingle  : single;
    function    ReadDouble  : double;
    function    ReadStringAZT : string;
    function    ReadLongString : string;
    procedure   ReadBuf(buf : pointer ; len : integer) ;
    property    Buf : pTArrayOfByte read FMemBuf write SetMemBuf;
    property    Size : integer read FSize write FSize;
    property    Pos : integer read FIntPos write FIntPos;
  end;

function ShowBuffer(buf : pointer ; len : integer) : string;
implementation

{$R-}

uses
  SysUtils,
  Forms, { pour ShowMessage }
  Classes;

const
  ENDOFSTRING      = #0;
  // To speed up a little, we use constant whereas SizeOf(thetype)
  SIZEOF_INTEGER   = SizeOf(integer);
  SIZEOF_SINGLE    = SizeOf(single);
  SIZEOF_DOUBLE    = SizeOf(double);
  SIZEOF_BOOLEAN   = SizeOf(byte);
  SIZEOF_CHAR      = SizeOf(char);
  SIZEOF_BYTE      = SizeOf(byte);
  SIZEOF_WORD      = SizeOf(word);
  SIZEOF_SMALLINT  = SizeOf(SmallInt);
  SIZEOF_SHORTINT  = SizeOf(ShortInt);
  SIZEOF_CARDINAL  = SizeOf(Cardinal);
  SIZEOF_STRING    = -1;


{$O+}
constructor TWriteLinearMemory.Create;
begin
  inherited;
  FMemBuf := nil;
  FSize := 0;
  FIntPos := 0;
  FAutoGrow := true;
end;

destructor TWriteLinearMemory.Destroy;
begin
  inherited;
  if (FMemBuf <> nil) then
    FreeMem(FMemBuf);
  FMemBuf := nil;
end;

procedure TWriteLinearMemory.SetSize(s : integer);
//var
//  newptr : pTArrayOfByte;
//  i : integer;
begin
  if (FSize = 0) and (s <> 0) then
  begin
    // premiere affectation de la taille
    GetMem(FMemBuf, s);
    FIntPos := 0;
    FSize := s;
  end
  else
  if (s < FSize) then
  begin
    // Rduction du buffer
    FSize := s;
  end
  else
  if (s > FSize) then
  begin
    // Augmentation du buffer
{    MyGetMem(newptr, s); // Alloc d'un nouveau buffer
    move(FMemBuf^, newptr^, FSize); // Copie du buffer
    MyFreeMem(FMemBuf); // Dalloc de l'ancien buffer
    i := integer(FPos) - integer(FMemBuf);
    FMemBuf := newptr;}
    Reallocmem(FMemBuf, s);
  end;
//  FAutoGrow := false;  // jf 15/02/2001 
end;

procedure TWriteLinearMemory.WriteInteger(i : integer);
begin
  WriteBuf(@i, SIZEOF_INTEGER);
end;

procedure TWriteLinearMemory.WriteSingle(s : single);
begin
  WriteBuf(@s, SIZEOF_SINGLE);
end;

procedure TWriteLinearMemory.WriteDouble(d : double);
begin
  WriteBuf(@d, SIZEOF_DOUBLE);
end;

procedure TWriteLinearMemory.WriteChar(c : char);
begin
  WriteBuf(@c, SIZEOF_CHAR);
end;

procedure TWriteLinearMemory.WriteStringAZT(s : string);
var
  p : PCHAR;
  l : integer;
begin
  p := PCHAR(s);
  l := Length(s);
  WriteBuf(p, l);
  WriteChar(ENDOFSTRING);
end;

Procedure TWriteLinearMemory.WriteLongString(s: string);   // Write string, even if there is some #0 in it.
var
  l : integer;
  i : integer;
begin
  l := Length(s);
  WriteInteger(l); // 4 premier octets : la longueur de la chaine
  for i := 1 to l do
  begin
    WriteChar(s[i]); // la chaine
  end;
end;


procedure TWriteLinearMemory.WriteByte(b : byte); // 0..255
begin
  WriteBuf(@b, SIZEOF_BYTE);
end;

procedure TWriteLinearMemory.WriteShortInt(si : ShortInt); // -128..127
begin
  WriteBuf(@si, SIZEOF_SHORTINT);
end;

procedure TWriteLinearMemory.WriteWord(w : word); // 0..65535
begin
  WriteBuf(@w, SIZEOF_WORD);
end;

procedure TWriteLinearMemory.WriteSmallInt(si : SmallInt); // -32768..32767
begin
  WriteBuf(@si, SIZEOF_SMALLINT);
end;

procedure TWriteLinearMemory.WriteCardinal(c : Cardinal); // 0..2^32-1
begin
  WriteBuf(@c, SIZEOF_CARDINAL);
end;

procedure TWriteLinearMemory.WriteRawString(s : string);
var
  p : PCHAR;
  l : integer;
begin
  p := PCHAR(s);
  l := Length(s);
  WriteBuf(p, l);
end;

procedure TWriteLinearMemory.WriteBuf(buf : pointer ; len : integer);
begin
  if (FAutoGrow) then
  begin
    if (FMemBuf = nil) then
    begin
      // buffer pas allou. On cr un buffer de la taille de la donne a crire
{      GetMem(FMemBuf, len);
      FSize := len;
      FIntPos := 0;}
    end;
    // Rallocation du buffer
    ReallocMem(FMemBuf, FSize + len);
    FSize := FSize + Len;
  end;
  try
//    Assert(not (FIntPos >= FSize), 'Buffer trop petit');
    if (FIntPos >= FSize) then
    begin
    end;

    move(buf^, FMemBuf^[FIntPos], len);
  except
  end;
  inc(FIntPos, len);
end;

procedure TWriteLinearMemory.SaveToFile(filename : string);
var
  fd : THANDLE;
  err : integer; 
begin
  fd := CreateFile(PCHAR(FileName),
                   GENERIC_WRITE,
                   FILE_SHARE_READ,
                   nil,
                   CREATE_ALWAYS,
                   FILE_ATTRIBUTE_NORMAL,
                   0);
  if (fd <> INVALID_HANDLE_VALUE) then
  begin
    AppendToFile(fd);
    CloseHandle(fd);
  end
  else
  begin
    err := GetLastError;
    Raise EWriteError.Create(
                      Format('Unable to open file for writing (err %d:%s)',
                      [err, SysErrorMessage(err)]));
  end;
end;

procedure TWriteLinearMemory.AppendToFile(fd : THANDLE);
var
  nEcrit : integer;
  err : integer;
begin
  if (WriteFile(fd, FMemBuf^, FSize, nEcrit, nil) = false) then
  begin
    err := GetLastError;
    Raise EWriteError.Create(
                      Format('Unable to write correctly to file (err %d:%s)',
                      [err, SysErrorMessage(err)]));
  end;
end;

{**
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************
 **}

constructor TReadLinearMemory.Create;
begin
  inherited;
  FMemBuf := nil;
  FSize := 0;
  FIntPos := 0;
end;

destructor TReadLinearMemory.Destroy;
begin
  inherited;
end;

procedure TReadLinearMemory.SetMemBuf(buf : pTArrayOfByte);
begin
  FMemBuf := buf;
  FIntPos := 0;
end;


function    TReadLinearMemory.ReadInteger : integer;
var
  i : integer;
begin
  ReadBuf(@i, SIZEOF_INTEGER);
  result := i;
end;

function    TReadLinearMemory.ReadSingle  : single;
var
  s : single;
begin
  ReadBuf(@s, SIZEOF_SINGLE);
  result := s;
end;

function    TReadLinearMemory.ReadDouble  : double;
var
  d : double;
begin
  ReadBuf(@d, SIZEOF_DOUBLE);
  result := d;
end;

function    TReadLinearMemory.ReadByte    : byte;
var
  b : byte;
begin
  ReadBuf(@b, SIZEOF_BYTE);
  result := b;
end;


function    TReadLinearMemory.ReadChar    : char;
var
  c : char;
begin
  ReadBuf(@c, SIZEOF_CHAR);
  result := c;
end;

function    TReadLinearMemory.ReadShortInt: shortint;
var
  si : ShortInt;
begin
  ReadBuf(@si, SIZEOF_SHORTINT);
  result := si;
end;

function    TReadLinearMemory.ReadWord    : word;
var
  w : Word;
begin
  ReadBuf(@w, SIZEOF_WORD);
  result := w;
end;

function    TReadLinearMemory.ReadSmallInt: SmallInt;
var
  si : SmallInt;
begin
  ReadBuf(@si, SIZEOF_SMALLINT);
  result := si;
end;

function    TReadLinearMemory.ReadCardinal: Cardinal;
var
  c : Cardinal;
begin
  ReadBuf(@c, SIZEOF_CARDINAL);
  result := c;
end;

function    TReadLinearMemory.ReadLongString : string;
var
  l : integer;
  i : integer;
  s : string;
begin
  s := '';
  l := ReadInteger; // Longueur de la chaine
  for i := 1 to l do
  begin
    s := s + ReadChar;
  end;
  result := s;
end;

function    TReadLinearMemory.ReadStringAZT : string;
var
  s : PCHAR;
begin
  s := pointer(@(FMemBuf^[FIntPos]));
  inc(FIntPos, StrLen(s) + 1);
  result := s;
end;

procedure   TReadLinearMemory.ReadBuf(buf : pointer ; len : integer);
begin
  move(FMemBuf^[FIntPos], buf^, len);
  inc(FIntPos, len);
end;

function ShowBuffer(buf : pointer ; len : integer) : string;
var
  i : integer;
  s : string;
begin
  s := '';
  for i := 0 to len do
  begin
    s := s + IntToHex(byte(pTArrayOfByte(buf)^[i]), 2);
  end;
  result := s;
end;
{$O-}
end.

