unit LbSpeedButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TLbColorStyle = (lcsCustom, lcsGold, lcsChrome, lcsBlue, lcsRed, lcsUltraFlat);
  TLbSpeedButtonLayout = (blGlyphLeft, blGlyphRight, blGlyphTop, blGlyphBottom);
  TLbSpeedButtonStyle = (bsNormal, bsEncarta, bsModern);

  TLbSpeedButton = class(TGraphicControl)
  private
    FAlignment: TAlignment;
    FAllowAllUp: boolean;
    FCaption: TCaption;
    FColorWhenDown: TColor;
    FDown: boolean;
    FFlat: boolean;
    FGlyph: TBitmap;
    FGroupIndex: integer;
    FHotTrackColor: TColor;
    FLayout: TLbSpeedButtonLayout;
    FLightColor: TColor;
    FNumGlyphs: integer;
    FShadowColor: TColor;
    FStyle: TLbSpeedButtonStyle;
    FTransparent: boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseExit: TNotifyEvent;
    FDummyStyle: TLbColorStyle;

    bCursorOnButton: boolean;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  protected
    procedure Paint; override;

    procedure SetAlignment(fNew: TAlignment);
    procedure SetFlat(fNew: boolean);
    procedure SetColorWhenDown(fNew: TColor);
    procedure SetColorStyle(fNew: TLbColorStyle);
    procedure SetDown(fNew: boolean);
    procedure SetCaption(const fNew: TCaption);
    procedure SetGlyph(fNew: TBitmap);
    procedure SetGroupIndex(fNew: integer);
    procedure SetLayout(fNew: TLbSpeedButtonLayout);
    procedure SetLightColor(fNew: TColor);
    procedure SetNumGlyphs(fNew: integer);
    procedure SetShadowColor(fNew: TColor);
    procedure SetStyle(fNew: TLbSpeedButtonStyle);
    procedure SetTransparent(fNew: boolean);

    procedure DoMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure DoMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  published
    property Align;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property AllowAllUp: boolean read FAllowAllUp write FAllowAllUp default false;
    property Caption: TCaption read FCaption write SetCaption;
    property Color;
    property ColorWhenDown: TColor read FColorWhenDown write SetColorWhenDown default clNone;
    property ColorStyle: TLbColorStyle read FDummyStyle write SetColorStyle default lcsCustom;
    property Down: boolean read FDown write SetDown default false;
    property Enabled;
    property Flat: boolean read FFlat write SetFlat default false;
    property Font;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property GroupIndex: integer read FGroupIndex write SetGroupIndex default 0;
    property HotTrackColor: TColor read FHotTrackColor write FHotTrackColor default clNone;
    property Hint;
    property Layout: TLbSpeedButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property LightColor: TColor read FLightColor write SetLightColor default clWhite;
    property NumGlyphs: integer read FNumGlyphs write SetNumGlyphs default 0;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShadowColor: TColor read FShadowColor write SetShadowColor default clBlack;
    property ShowHint;
    property Style: TLbSpeedButtonStyle read FStyle write SetStyle default bsNormal;
    property Transparent: boolean read FTransparent write SetTransparent default false;
    property Visible;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

{$R LBSPEEDBUTTON.DCR}

{##############################################################################}

procedure DrawColorFade(aCanvas: TCanvas; StartColor, StopColor: TColor; iLeft, iTop, iRight, iBottom: integer);
var
   iCounter, iBuffer, iFillStep: integer;
   bR1, bG1, bB1, bR2, bG2, bB2: byte;
   aColor1, aColor2: LongInt;
   dCurrentR, dCurrentG, dCurrentB, dRStep, dGStep, dBStep: double;
   aOldStyle: TPenStyle;
   iHeight, iDrawBottom: integer;

begin
   iHeight := (iBottom - iTop);
   aOldStyle := aCanvas.Pen.Style; aCanvas.Pen.Style := psClear;
   aColor1 := ColorToRGB(StartColor); bR1 := GetRValue(aColor1); bG1 := GetGValue(aColor1); bB1 := GetBValue(aColor1);
   aColor2 := ColorToRGB(StopColor);  bR2 := GetRValue(aColor2); bG2 := GetGValue(aColor2); bB2 := GetBValue(aColor2);
   dCurrentR := bR1; dCurrentG := bG1; dCurrentB := bB1;
   dRStep := (bR2-bR1) / 31; dGStep := (bG2-bG1) / 31; dBStep := (bB2-bB1) / 31;

   iFillStep := (iHeight div 31) + 1;
   for iCounter := 0 to 31 do
   begin
      iBuffer := iCounter * iHeight div 31;
      aCanvas.Brush.Color := rgb(trunc(dCurrentR), trunc(dCurrentG), trunc(dCurrentB));
      dCurrentR := dCurrentR + dRStep; dCurrentG := dCurrentG + dGStep; dCurrentB := dCurrentB + dBStep;
      iDrawBottom := iTop + iBuffer + iFillStep; if iDrawBottom > iBottom then iDrawBottom := iBottom;
      aCanvas.FillRect(Rect(iLeft, iTop + iBuffer, iRight, iDrawBottom));
   end;
   aCanvas.Pen.Style := aOldStyle;
end;

{##############################################################################}

procedure Register;
begin
  RegisterComponents('LB', [TLbSpeedButton]);
end;

{##############################################################################}

constructor TLbSpeedButton.Create(aOwner: TComponent);
begin
   inherited;

   ControlStyle := ControlStyle + [csSetCaption];
   ControlStyle := ControlStyle - [csOpaque];

   Height := 23;
   Width := 100;

   bCursorOnButton := false;
   FLightColor := clWhite;
   FShadowColor := clBlack;
   FColorWhenDown := clNone;
   FHotTrackColor := clNone;
   FAlignment := taLeftJustify;
   FDummyStyle := lcsCustom;

   FGlyph := TBitmap.Create;
end;

{##############################################################################}

destructor TLbSpeedButton.Destroy;
begin
   FGlyph.Free;
   inherited;
end;

{##############################################################################}

procedure TLbSpeedButton.SetAlignment(fNew: TAlignment);
begin
   FAlignment := fNew;
   if FTransparent then invalidate else paint;  // Could have generally used invalidate, but why make it flicker if it is not transparent?
end;

{##############################################################################}

procedure TLbSpeedButton.SetCaption(const fNew: TCaption);
begin
   FCaption := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetColorWhenDown(fNew: TColor);
begin
   FColorWhenDown := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetColorStyle(fNew: TLbColorStyle);
begin
   case fNew of
      lcsGold:       begin Color := $0000C0C0;  LightColor := clYellow;    ShadowColor := clOlive;    ColorWhenDown := clNone;     HotTrackColor := clNone;     FFlat := False; FStyle := bsModern;  end;
      lcsChrome:     begin Color := clSilver;   LightColor := clWhite;     ShadowColor := clGray;     ColorWhenDown := clNone;     HotTrackColor := clNone;     FFlat := False; FStyle := bsModern;  end;
      lcsBlue:       begin Color := $00FF8000;  LightColor := clAqua;      ShadowColor := clBlue;     ColorWhenDown := clNone;     HotTrackColor := clNone;     FFlat := False; FStyle := bsModern;  end;
      lcsRed:        begin Color := clRed;      LightColor := $00C0C0FF;   ShadowColor := $000000C0;  ColorWhenDown := clNone;     HotTrackColor := clNone;     FFlat := False; FStyle := bsModern;  end;
      lcsUltraFlat:  begin Color := clBtnFace;  LightColor := $00B59284;   ShadowColor := $00B59284;  ColorWhenDown := $00B59284;  HotTrackColor := $00DED3D6;  FFlat := True;  FStyle := bsNormal;  end;
   end;
   Paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetDown(fNew: boolean);
var
   i: integer;

begin
   if FDown = fNew then exit;

   // If grouped, set all siblings to down=false
   if GroupIndex <> 0 then
      for i := 0 to Parent.ControlCount-1 do
         if Parent.Controls[i] is TLbSpeedButton then
            if TLbSpeedButton(Parent.Controls[i]).GroupIndex = GroupIndex then
               if TLbSpeedButton(Parent.Controls[i]) <> self then
                  TLbSpeedButton(Parent.Controls[i]).Down := false;

   FDown := fNew;

   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetFlat(fNew: boolean);
begin
   FFlat := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetGlyph(fNew: TBitmap);
begin
   if fNew <> nil then
   begin
      FGlyph.Assign(fNew);
      if fNew.Height <> 0 then FNumGlyphs := fNew.Width div fNew.Height else FNumGlyphs := 0;
   end
   else
   begin
      FGlyph.Height := 0;
      FNumGlyphs := 0;
   end;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetGroupIndex(fNew: integer);
begin
   FGroupIndex := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetLayout(fNew: TLbSpeedButtonLayout);
begin
   FLayout := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetLightColor(fNew: TColor);
begin
   FLightColor := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetNumGlyphs(fNew: integer);
begin
   FNumGlyphs := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetShadowColor(fNew: TColor);
begin
   FShadowColor := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetStyle(fNew: TLbSpeedButtonStyle);
begin
   if GetDeviceCaps(Canvas.Handle, BITSPIXEL) <= 8 then if not (csDesigning in ComponentState) then fNew := bsNormal;
   FStyle := fNew;
   if fNew = bsModern then FAlignment := taCenter;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetTransparent(fNew: boolean);
begin
   FTransparent := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.Paint;
var
   iCaptionHeight, iCaptionWidth, iGlyphHeight, iGlyphWidth: integer;
   iGlyphIndex: integer;
   iOffset: integer;
   sDrawCaption: string;
   clBackColor: TColor;
   iCapX, iCapY, iGlX, iGlY: integer;
   wR, wG, wB: word;

   procedure DrawGlyph(iDestLeft, iDestTop, iSrcLeft, iSrcTop, iWidth, iHeight: integer);  // transparent draw
   var
      aPicture: TPicture;

   begin
      aPicture := TPicture.Create;
      try aPicture.Bitmap.Assign(fGlyph); except end;
      aPicture.Bitmap.Width := iWidth;
      aPicture.Bitmap.Height := iHeight;
      aPicture.Bitmap.Canvas.Draw(-iSrcLeft, -iSrcTop, FGlyph);
      aPicture.Graphic.Transparent := true;
      Canvas.Draw(iDestLeft, iDestTop, aPicture.Graphic);
      aPicture.Free;
   end;

begin
   if not (Visible or (csDesigning in ComponentState)) or (csLoading in ComponentState) then exit;
   iOffset := 0; if Down then if FStyle in [bsNormal, bsModern] then iOffset := 1;

   // Background
   clBackColor := colortorgb(Color);
   if bCursorOnButton then if FHotTrackColor <> clNone then clBackColor := FHotTrackColor;
   if Down then if FColorWhenDown <> clNone then clBackColor := FColorWhenDown;

   if not FTransparent then
   begin
      Canvas.Brush.Color := clBackColor;
      if FStyle <> bsModern then Canvas.Rectangle(-1, -1, Width+1, Height+1)
      else
      begin
         DrawColorFade(Canvas, FLightColor, clBackColor, 2, 2, Width - 2, Height div 4 + iOffset);
         DrawColorFade(Canvas, clBackColor, FLightColor, 2, Height div 4 + iOffset, Width - 2, Height - 1);
      end;
   end;

   Canvas.Brush.Style := bsclear;

   // Border
   if Style <> bsModern then
   begin
      if Enabled and (not Flat or bCursorOnButton or (csDesigning in ComponentState) or FDown) then
      begin
         with Canvas do
         begin
            if FDown then Pen.Color := FShadowColor else Pen.Color := FLightColor;
            MoveTo(0, Height-1);
            LineTo(0, 0);
            LineTo(Width-1, 0);
            if FDown then Pen.Color := FLightColor else Pen.Color := FShadowColor;
            LineTo(Width-1, Height-1);
            LineTo(0, Height-1);
         end;
      end;
   end
   else
   begin
      with Canvas do
      begin
         Pen.Color := clBackColor; if Down then Pen.Color := FShadowColor;
         Rectangle(1, 1, Width-1, Height-1);
         Pen.Color := FShadowColor;
         RoundRect(0, 0, Width, Height, 6, 6);
      end;
   end;

   // Prepare layout
   Canvas.Font := Font;
   if Down then if FStyle = bsEncarta then Canvas.Font.Style := Canvas.Font.Style + [fsBold];

   if not FGlyph.Empty then
   begin
      if FLayout = blGlyphLeft then sDrawCaption := ' ' + FCaption else sDrawCaption := FCaption + ' ';
      if sDrawCaption = ' ' then sDrawCaption := '';
   end
   else sDrawCaption := FCaption;

   iCaptionHeight := Canvas.TextHeight(sDrawCaption);
   iCaptionWidth := Canvas.TextWidth(sDrawCaption);
   iGlyphHeight := FGlyph.Height;
   if FNumGlyphs <> 0 then iGlyphWidth := FGlyph.Width div FNumGlyphs else iGlyphWidth := 0;
   iGlyphIndex := 0; if not Enabled then iGlyphIndex := iGlyphWidth;

   // Text + Glyph
   iCapX := 0; iCapY := 0; iGlX := 0; iGlY := 0; // Just to get rid of these warnings...

   if FLayout = blGlyphLeft then
   begin
      iCapY := (Height - iCaptionHeight) div 2 + iOffset; iGlY := (Height - iGlyphHeight) div 2 + iOffset;
      case FAlignment of
         taLeftJustify:    begin iCapX := 4 + iGlyphWidth + iOffset; iGlX := 4 + iOffset; end;
         taRightJustify:   begin iCapX := Width - 4 - iCaptionWidth + iOffset; iGlX := Width - 4 - iCaptionWidth - iGlyphWidth + iOffset; end;
         taCenter:         begin iCapX := (Width - iCaptionWidth - iGlyphWidth) div 2 + iGlyphWidth + iOffset; iGlX := (Width - iCaptionWidth - iGlyphWidth) div 2 + iOffset; end;
      end;
   end
   else
   if FLayout = blGlyphRight then
   begin
      iCapY := (Height - iCaptionHeight) div 2 + iOffset; iGlY := (Height - iGlyphHeight) div 2 + iOffset;
      case FAlignment of
         taLeftJustify:    begin iCapX := 4 + iOffset; iGlX := 4 + iCaptionWidth + iOffset; end;
         taRightJustify:   begin iCapX := Width - 4 - iCaptionWidth - iGlyphWidth + iOffset; iGlX := Width - 4 - iGlyphWidth + iOffset; end;
         taCenter:         begin iCapX := (Width - iCaptionWidth - iGlyphWidth) div 2 + iOffset; iGlX := (Width - iCaptionWidth - iGlyphWidth) div 2 + iCaptionWidth + iOffset; end;
      end;
   end
   else
   if FLayout = blGlyphTop then
   begin
      iCapY := (Height - iCaptionHeight - iGlyphHeight) div 2 + iGlyphHeight + iOffset; iGlY := (Height - iCaptionHeight - iGlyphHeight) div 2 + iOffset;
      case FAlignment of
         taLeftJustify:    begin iCapX := 4 + iOffset; iGlX := 4 + iOffset; end;
         taRightJustify:   begin iCapX := Width - 4 - iCaptionWidth + iOffset; iGlX := Width - 4 - iGlyphWidth + iOffset; end;
         taCenter:         begin iCapX := (Width - iCaptionWidth) div 2 + iOffset; iGlX := (Width - iGlyphWidth) div 2 + iOffset; end;
      end;
   end
   else
   if FLayout = blGlyphBottom then
   begin
      iCapY := (Height - iCaptionHeight - iGlyphHeight) div 2 + iOffset; iGlY := (Height - iCaptionHeight - iGlyphHeight) div 2 + iCaptionHeight + iOffset;
      case FAlignment of
         taLeftJustify:    begin iCapX := 4 + iOffset; iGlX := 4 + iOffset; end;
         taRightJustify:   begin iCapX := Width - 4 - iCaptionWidth + iOffset; iGlX := Width - 4 - iGlyphWidth + iOffset; end;
         taCenter:         begin iCapX := (Width - iCaptionWidth) div 2 + iOffset; iGlX := (Width - iGlyphWidth) div 2 + iOffset; end;
      end;
   end;

   if not Enabled then Canvas.Font.Color := clGray;
   Canvas.TextRect(Rect(1, 1, Width-1, Height-1), iCapX, iCapY, sDrawCaption);
   DrawGlyph(iGlX, iGlY, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
end;

{##############################################################################}

procedure TLbSpeedButton.DoMouseEnter(var Msg: TMessage);
begin
   bCursorOnButton := true;
   if FTransparent then invalidate else paint;
   if assigned(FOnMouseEnter) then FOnMouseEnter(self);
end;

{##############################################################################}

procedure TLbSpeedButton.DoMouseLeave(var Msg: TMessage);
begin
   bCursorOnButton := false;
   if FTransparent then invalidate else paint;
   if assigned(FOnMouseExit) then FOnMouseExit(self);
end;

{##############################################################################}

procedure TLbSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if GroupIndex = 0 then Down := true
   else
   begin
      if Down then
      begin
         if FAllowAllUp then Down := false;
      end
      else
         Down := true;
   end;
end;

{##############################################################################}

procedure TLbSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if GroupIndex = 0 then Down := false;
end;

{##############################################################################}

end.
