{$I PIETOOLS.INC}
{$R-} {RangeCheck unbedingt ausschalten!}
{ Autor: Ingolf Pietschmann.
  Dieser Quelltext ist Freeware. Die Verwendung und Weitergabe dieser Sourcen zu
  privaten nicht kommerziellen Zwecken ist ausdrcklich erwnscht.
  Die Verwendung zu kommerziellen Zwecken ist nur mit Erlaubnis des Autors
  gestattet. Den Autor knnen Sie unter "Support@Pie-Tools.de" erreichen.

  These sources are freeware. The usage and distribution of these sources for
  private, not commercial purposes is explicit desired.
  The usage for commercial purposes is only permitted in agreement of the author.
  The author can be reached by "Support@Pie-Tools.de".
}
unit PieComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls,
  Piezln1, DsgnIntf, ExtDlgs, imgList;

type
  {PieComboBoxStrings}
  TPieComboBoxEintrag = class(TObject)
  public
    FBitmap: TBitmap;
    FColor: TColor;
    FFont: TFont;
    FHelpContext: THelpContext;
    FHint: string;
  END;

  PPieComboBoxItem = ^TPieComboBoxItem;
  TPieComboBoxItem = RECORD
    FString: string;
    FPieComboBoxEintrag: TPieComboBoxEintrag;
  END;

  PPieComboBoxListe = ^TPieComboBoxListe;
  TPieComboBoxListe = array[0..MaxListSize] of TPieComboBoxItem;

  TPieComboBox = class;

  TPieComboBoxStrings = class(TStrings)
  private
    FList: PPieComboBoxListe;
    FComboBox: TPieComboBox;
    FCount: Integer;
    FCapacity: Integer;
    FSorted: Boolean;
    FDuplicates: TDuplicates;
    FOnChange: TNotifyEvent;
    FOnChanging: TNotifyEvent;
    procedure ExchangeItems(Index1, Index2: Integer);
    procedure Grow;
    procedure QuickSort(L, R: Integer);
    procedure InsertItem(Index: Integer; const S: string);
    procedure SetSorted(Value: Boolean);
    procedure ReadBitmapData(Stream: TStream);
    procedure ReadData(Reader: TReader);
    procedure WriteBitmapData(Stream: TStream);
    procedure WriteData(Writer: TWriter);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Changed; virtual;
    procedure Changing; virtual;
    function Get(Index: Integer): string; override;
    function GetCapacity: Integer; override;
    function GetCount: Integer; override;
    function GetBitmap(Index: Integer): TBitmap; virtual;
    function GetColor(Index: Integer): TColor; virtual;
    function GetFont(Index: Integer): TFont; virtual;
    function GetHelpContext(Index: Integer): THelpContext; virtual;
    function GetHint(Index: Integer): string; virtual;
    function GetPieComboBoxEintrag(Index: Integer): TPieComboBoxEintrag; virtual;
    procedure Put(Index: Integer; const S: string); override;
    procedure PutPieComboBoxEintrag(Index: Integer; APieComboBoxEintrag: TPieComboBoxEintrag); virtual;
    procedure SetBitmap(Index: Integer; ABitmap: TBitmap); virtual;
    procedure SetColor(Index: Integer; AColor: TColor); virtual;
    procedure SetFont(Index: Integer; AFont: TFont); virtual;
    procedure SetHelpContext(Index: Integer; AHelpContext: THelpContext); virtual;
    procedure SetHint(Index: Integer; AHint: string); virtual;
    procedure SetCapacity(NewCapacity: Integer); override;
    procedure SetUpdateState(Updating: Boolean); override;
    property PieComboBoxEintraege[Index: Integer]: TPieComboBoxEintrag read GetPieComboBoxEintrag write PutPieComboBoxEintrag;
  public       {PieComboBoxStrings}
    constructor Create(AOwner: TPieComboBox);
    destructor Destroy; override;
    function Add(const S: string): Integer; override;
    procedure AddItems(ItemsNeu: TPieComboBoxStrings); virtual;
    procedure Assign(Source: TPersistent); override;
    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    procedure Exchange(Index1, Index2: Integer); override;
    function Find(const S: string; var Index: Integer): Boolean; virtual;
    function IndexOf(const S: string): Integer; override;
    procedure Insert(Index: Integer; const S: string); override;
    procedure Sort; virtual;
    property Colors[Index: Integer]: TColor read GetColor write SetColor;
    property Hints[Index: Integer]: string read GetHint write SetHint;
    property HelpContexts[Index: Integer]: THelpContext read GetHelpContext write SetHelpContext;
    property Fonts[Index: Integer]: TFont read GetFont write SetFont;
    property Bitmaps[Index: Integer]: TBitmap read GetBitmap write SetBitmap;
  published
    property Duplicates: TDuplicates read FDuplicates write FDuplicates;
    property Sorted: Boolean read FSorted write SetSorted;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnChanging: TNotifyEvent read FOnChanging write FOnChanging;
  end;

  {PieComboBox}
  TBitmapStyle = (bscLeft, bscRight, bscCenter, bscStretch);
  TItemHeightStyle = (ihcFix, ihcFont, ihcBitmap, ihcFontBitmap);
  TBackGroundStyle = (bgscNone, bgscCenter, bgscStretch, bgscPattern);

  EInvalidTabStop = exception;

  TPieComboBox = class(TWinControl)
  private
    FItems: TPieComboBoxStrings;
    FCanvas: TCanvas;
    FSorted: Boolean;
    FStyle: TComboBoxStyle;
    FItemHeight: Integer;
    FMaxLength: Integer;
    FDropDownCount: Integer;
    FEditHandle: HWnd;
    FListHandle: HWnd;
    FEditInstance: Pointer;
    FListInstance: Pointer;
    FDefEditProc: Pointer;
    FDefListProc: Pointer;
    FIsFocused: Boolean;
    FFocusChanged: Boolean;
    FSaveItems: TPieComboBoxStrings;
    FOnChange: TNotifyEvent;
    FOnDropDown: TNotifyEvent;
    FOnDrawItem: TDrawItemEvent;
    FOnMeasureItem: TMeasureItemEvent;
    FAnyColors: Boolean;
    FAnyFonts: Boolean;
    FBitmapStyle: TBitmapStyle;
    FItemHeightStyle: TItemHeightStyle;
    FLineByTabStops: Boolean;
    FBoldStyle: Boolean;
    FBackground: TPicture;
    FBackGroundStyle: TBackGroundStyle;
    FVersion: string;
    procedure AdjustDropDown;
    procedure EditWndProc(var Message: TMessage);
    function DrawItemBitmap(Index: integer; ARect: TRect): TRect;
    function GetDroppedDown: Boolean;
    function GetItemIndex: Integer;
    function GetSelLength: Integer;
    function GetSelStart: Integer;
    function GetSelText: string;
    procedure ListWndProc(var Message: TMessage);
    procedure SetDroppedDown(Value: Boolean);
    procedure SetItems(Value: TPieComboBoxStrings);
    procedure SetItemIndex(Value: Integer);
    procedure SetItemProperties;
    procedure SetSelLength(Value: Integer);
    procedure SetSelStart(Value: Integer);
    procedure SetSelText(const Value: string);
    procedure SetSorted(Value: Boolean);
    function  GetItemHeight: Integer;
    procedure SetItemHeight(Value: Integer);
    procedure SetMaxLength(Value: Integer);
    procedure UpdateHint(Index: Integer);
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure CMCancelMode(var Message: TCMCancelMode); message CM_CANCELMODE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMDrawItem(var Message: TWMDrawItem); message WM_DRAWITEM;
    procedure WMMeasureItem(var Message: TWMMeasureItem); message WM_MEASUREITEM;
    procedure WMDeleteItem(var Message: TWMDeleteItem); message WM_DELETEITEM;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure DrawBoldStyle(Index: integer; ARect: TRect; State: TOwnerDrawState);
    procedure SetAnyColors(Value: Boolean);
    procedure SetAnyFonts(Value: Boolean);
    procedure SetBitmapStyle(AStyle: TBitmapStyle);
    procedure SetBoldStyle(AStyle: Boolean);
    procedure SetItemHeightStyle(AStyle: TItemHeightStyle);
    procedure SetBackground(Value: TPicture);
    procedure SetBackGroundStyle(Value: TBackGroundStyle);
    procedure BackGroundChange(Sender: TObject);
    procedure WMMove(var Message: TWMMove); message WM_MOVE;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMPaint(var Message: TWmPaint); message WM_Paint;
    procedure PaintBackground(R: TRect; Edit_malen: Boolean);
    procedure SetLineByTabStops(Value: Boolean);
    function GetHCVisible: Boolean;
    procedure SetHCVisible(Value: Boolean);
    function GetHCS: THeaderSections;
    procedure SetHCS(Value: THeaderSections);
    function GetHCImages: TCustomImageList;
    procedure SetHCImages(Value: TCustomImageList);
    function GetHCStyle: THeaderStyle;
    procedure SetHCStyle(Value: THeaderStyle);
    function GetHCFont: TFont;
    procedure SetHCFont(Value: TFont);
    function GetOnSectionClick: TSectionNotifyEvent;
    procedure SetOnSectionClick(Value: TSectionNotifyEvent);
    procedure HeaderSectionResize(HeaderControl: THeaderControl; Section: THeaderSection);
  protected
    Header: THeaderControl;
    procedure ComboWndProc(var Message: TMessage; ComboWnd: HWnd;
      ComboProc: Pointer); virtual;
    procedure WndProc(var Message: TMessage); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure Loaded; override;
    procedure DrawItem(Index: Integer; Rect: TRect;
      State: TOwnerDrawState); virtual;
    procedure MeasureItem(Index: Integer; var AHeight: Integer); virtual;
    procedure Change; dynamic;
    procedure DropDown; dynamic;
    procedure SetStyle(Value: TComboBoxStyle); virtual;
    property EditHandle: HWnd read FEditHandle;
    property ListHandle: HWnd read FListHandle;
    property OnDragDrop;
    property OnDragOver;
    property OnMouseDown;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure SelectAll;
    procedure Edit_aufrufen;
    function ItemAtPos(Pos: TPoint; Existing: Boolean): Integer;
    property Canvas: TCanvas read FCanvas;
    procedure CopyProp(Dest, Source: TPieComboBox);
    property DroppedDown: Boolean read GetDroppedDown write SetDroppedDown;
    property ItemIndex: Integer read GetItemIndex write SetItemIndex;
    property SelLength: Integer read GetSelLength write SetSelLength;
    property SelStart: Integer read GetSelStart write SetSelStart;
    property SelText: string read GetSelText write SetSelText;
  published
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Text;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property AnyColors: boolean read FAnyColors write SetAnyColors default TRUE;
    property AnyFonts: boolean read FAnyFonts write SetAnyFonts default TRUE;
    property Background: TPicture read FBackground write SetBackground;
    property BackGroundStyle: TBackGroundStyle read FBackGroundStyle write SetBackGroundStyle;
    property BitmapStyle: TBitmapStyle read FBitmapStyle write SetBitmapStyle default bscLeft;
    property BoldStyle: Boolean read FBoldStyle write SetBoldStyle default FALSE;
    property DropDownCount: Integer read FDropDownCount write FDropDownCount default 8;
    property ItemHeight: Integer read GetItemHeight write SetItemHeight;
    property ItemHeightStyle: TItemHeightStyle read FItemHeightStyle
      write SetItemHeightStyle default ihcFix;
    property Items: TPieComboBoxStrings read FItems write SetItems;
    property MaxLength: Integer read FMaxLength write SetMaxLength default 0;
    property ParentColor default False;
    property Sorted: Boolean read FSorted write SetSorted default False;
    property Style: TComboBoxStyle read FStyle write SetStyle;
    property TabStop default True;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDropDown: TNotifyEvent read FOnDropDown write FOnDropDown;
    property OnDrawItem: TDrawItemEvent read FOnDrawItem write FOnDrawItem;
    property OnMeasureItem: TMeasureItemEvent read FOnMeasureItem write FOnMeasureItem;
    property OnSectionClick: TSectionNotifyEvent read GetOnSectionClick write SetOnSectionClick;
    property HeaderControl: Boolean read GetHCVisible write SetHCVisible default FALSE;
    property HeaderSections: THeaderSections read GetHCS write SetHCS;
    property HeaderImages: TCustomImageList read GetHCImages write SetHCImages;
    property HeaderStyle: THeaderStyle read GetHCStyle write SetHCStyle default hsButtons;
    property HeaderFont: TFont read GetHCFont write SetHCFont;
    property LineByTabStops: boolean read FLineByTabStops write SetLineByTabStops default FALSE;
    property Version: string read FVersion write FVersion;
  end;

  TPieComboBoxEditor = class(TComponentEditor)
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): string; override;
    procedure ExecuteVerb(Index: Integer); override;
  end;


  {PieItemsPropertyEditor}
  TPieComboBoxItemsfenster = class(TForm)
    Rechtspanel: TPanel;
    EditPanel: TPanel;
    ObenPanel: TPanel;
    ColorBtn: TBitBtn;
    FontBtn: TBitBtn;
    HintBtn: TBitBtn;
    ColorShape: TShape;
    Bevel1: TBevel;
    ZeilenLabel1: TLabel;
    Gesamtzeilenlabel1: TLabel;
    ZeilenLabel2: TLabel;
    Gesamtzeilenlabel2: TLabel;
    ColorDialog: TColorDialog;
    BitmapBtn: TBitBtn;
    Edit: TMemo;
    FontDialog: TFontDialog;
    ObenPanel3: TPanel;
    InsertBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    ChangeBtn: TBitBtn;
    Helplabel: TLabel;
    UT: TCheckBox;
    Bevel2: TBevel;
    AnyColors: TCheckBox;
    AnyFonts: TCheckBox;
    Bevel3: TBevel;
    UntenPanel: TPanel;
    LoeschBtn: TBitBtn;
    OkButton: TBitBtn;
    AbbruchButton: TBitBtn;
    OpenPicDialog: TOpenPictureDialog;
    IHStyle: TRadioGroup;
    BStyle: TRadioGroup;
    BitmapVorschau: TImage;
    BitBtn1: TBitBtn;
    BoldStyle: TCheckBox;
    HSGroup: TRadioGroup;
    LBTS: TCheckBox;
    HFButton: TBitBtn;
    HelpUpDown: TUpDown;
    HelpEdit: TLabel;
    BGButton: TBitBtn;
    BGStyle: TRadioGroup;
    Bevel4: TBevel;
    procedure InsertBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure ChangeBtnClick(Sender: TObject);
    procedure ColorBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EditEnter(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure FontBtnClick(Sender: TObject);
    procedure HintBtnClick(Sender: TObject);
    procedure LoeschBtnClick(Sender: TObject);
    procedure UTClick(Sender: TObject);
    procedure AnyColorsClick(Sender: TObject);
    procedure AnyFontsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BitmapBtnClick(Sender: TObject);
    procedure IHStyleClick(Sender: TObject);
    procedure BStyleClick(Sender: TObject);
    procedure BoldStyleClick(Sender: TObject);
    procedure LBTSClick(Sender: TObject);
    procedure HSGroupClick(Sender: TObject);
    procedure HFButtonClick(Sender: TObject);
    procedure HelpUpDownChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure BGButtonClick(Sender: TObject);
    procedure BGStyleClick(Sender: TObject);
  private
    { Private-Deklarationen }
    Zeile: Integer;
    ComboBox: TPieComboBox;
    Zieh_Item: Integer;
    HintDialog: TPieZeilenDialog;
    procedure ComboBoxClick(Sender: TObject);
    procedure ComboBoxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ComboBoxDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure Prop_zeigen;
  public
    { Public-Deklarationen }
  end;

implementation

{$R *.DFM}
USES Consts, PieHERK;

TYPE
  TSelection = record
    StartPos, EndPos: Integer;
  end;

function HasPopup(Control: TControl): Boolean;
begin
  Result := True;
  while Control <> nil do
    if TPieComboBox(Control).PopupMenu <> nil then Exit else Control := Control.Parent;
  Result := False;
end;


{ TPieComboBoxStrings }
constructor TPieComboBoxStrings.Create(AOwner: TPieComboBox);
begin
  inherited Create;
  FComboBox := AOwner;
  FList := NIL;
  FOnChange := nil;
  FOnChanging := nil;
  FCount := 0;
  SetCapacity(0);
end;

destructor TPieComboBoxStrings.Destroy;
VAR
  Index: Integer;
begin
  FOnChange := nil;
  FOnChanging := nil;
  FOR Index:=0 TO FCount-1 DO FList^[Index].FPieComboBoxEintrag.Free;
  if FCount <> 0 then Finalize(FList^[0], FCount);
  FComboBox := NIL;
  inherited Destroy;
  FCount := 0;
  SetCapacity(0);
end;

function TPieComboBoxStrings.Add(const S: string): Integer;
begin
  if not Sorted then
    Result := FCount
  else
    if Find(S, Result) then
      case Duplicates of
        dupIgnore: Exit;
        dupError: Error(SDuplicateString, 0);
      end;
  InsertItem(Result, S);
end;

procedure TPieComboBoxStrings.AddItems(ItemsNeu: TPieComboBoxStrings);
var
  I: Integer;
begin
  BeginUpdate;
  try
    for I := 0 to ItemsNeu.Count - 1 do begin
      Add(ItemsNeu.Strings[I]);
      Bitmaps[I] := ItemsNeu.Bitmaps[I];
      Colors[I] := ItemsNeu.Colors[I];
      Fonts[I] := ItemsNeu.Fonts[I];
      HelpContexts[I] := ItemsNeu.HelpContexts[I];
      Hints[I] := ItemsNeu.Hints[I];
    end;
  finally
    EndUpdate;
  end;
end;

procedure TPieComboBoxStrings.Assign(Source: TPersistent);
begin
  if Source is TPieComboBoxStrings then begin
    BeginUpdate;
    try
      Clear;
      AddItems(TPieComboBoxStrings(Source));
    finally
      EndUpdate;
    end;
    Exit;
  end;
  inherited Assign(Source);
end;

procedure TPieComboBoxStrings.Changed;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TPieComboBoxStrings.Changing;
begin
  if Assigned(FOnChanging) then FOnChanging(Self);
end;

procedure TPieComboBoxStrings.Clear;
begin
  if FCount <> 0 then begin
    Changing;
    Finalize(FList^[0], FCount);
    FCount := 0;
    SetCapacity(0);
    SendMessage(FComboBox.Handle, CB_RESETCONTENT, 0, 0);
    Changed;
  end;
end;

procedure TPieComboBoxStrings.DefineProperties(Filer: TFiler);

  function DoWrite: Boolean;
  begin
    if Filer.Ancestor <> nil then begin
      Result := True;
      if Filer.Ancestor is TPieComboBoxStrings then
        Result := not Equals(TPieComboBoxStrings(Filer.Ancestor))
    end
    else Result := Count > 0;
  end;

  function DoBitmapWrite: Boolean;
  begin
    if Filer.Ancestor <> nil then begin
      Result := True;
      if Filer.Ancestor is TPieComboBoxStrings then
        Result := TRUE;
    end
    else Result := Count > 0;
  end;

begin
  Filer.DefineProperty('PieComboBoxStrings', ReadData, WriteData, DoWrite);
  Filer.DefineBinaryProperty('PieComboBoxBitmaps', ReadBitmapData, WriteBitmapData, DoBitmapWrite);
end;

procedure TPieComboBoxStrings.Delete(Index: Integer);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;
  Finalize(FList^[Index]);
  Dec(FCount);
  if Index < FCount then
    System.Move(FList^[Index + 1], FList^[Index],
      (FCount - Index) * SizeOf(TPieComboBoxItem));
  SendMessage(FComboBox.Handle, CB_DELETESTRING, Index, 0);
  Changed;
end;

procedure TPieComboBoxStrings.Exchange(Index1, Index2: Integer);
begin
  if (Index1 < 0) or (Index1 >= FCount) then Error(SListIndexError, Index1);
  if (Index2 < 0) or (Index2 >= FCount) then Error(SListIndexError, Index2);
  Changing;
  ExchangeItems(Index1, Index2);
  Changed;
end;

procedure TPieComboBoxStrings.ExchangeItems(Index1, Index2: Integer);
var
  Temp: Integer;
  Item1, Item2: PPieComboBoxItem;
begin
  Item1 := @FList^[Index1];
  Item2 := @FList^[Index2];
  Temp := Integer(Item1^.FString);
  Integer(Item1^.FString) := Integer(Item2^.FString);
  Integer(Item2^.FString) := Temp;
  Temp := Integer(Item1^.FPieComboBoxEintrag);
  Integer(Item1^.FPieComboBoxEintrag) := Integer(Item2^.FPieComboBoxEintrag);
  Integer(Item2^.FPieComboBoxEintrag) := Temp;
end;

function TPieComboBoxStrings.Find(const S: string; var Index: Integer): Boolean;
var
  L, H, I, C: Integer;
begin
  Result := False;
  L := 0;
  H := FCount - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    C := AnsiCompareText(FList^[I].FString, S);
    if C < 0 then L := I + 1 else
    begin
      H := I - 1;
      if C = 0 then
      begin
        Result := True;
        if Duplicates <> dupAccept then L := I;
      end;
    end;
  end;
  Index := L;
end;

function TPieComboBoxStrings.Get(Index: Integer): string;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FString;
end;

function TPieComboBoxStrings.GetCapacity: Integer;
begin
  Result := FCapacity;
end;

function TPieComboBoxStrings.GetCount: Integer;
begin
  Result := FCount;
end;

function TPieComboBoxStrings.GetBitmap(Index: Integer): TBitmap;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FPieComboBoxEintrag.FBitmap;
{  Result := NIL;}
end;

function TPieComboBoxStrings.GetColor(Index: Integer): TColor;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FPieComboBoxEintrag.FColor;
end;

function TPieComboBoxStrings.GetFont(Index: Integer): TFont;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FPieComboBoxEintrag.FFont;
end;

function TPieComboBoxStrings.GetHelpContext(Index: Integer): THelpContext;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FPieComboBoxEintrag.FHelpContext;
end;

function TPieComboBoxStrings.GetHint(Index: Integer): string;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FPieComboBoxEintrag.FHint;
end;

function TPieComboBoxStrings.GetPieComboBoxEintrag(Index: Integer): TPieComboBoxEintrag;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FPieComboBoxEintrag;
end;

procedure TPieComboBoxStrings.Grow;
var
  Delta: Integer;
begin
  if FCapacity > 64 then Delta := FCapacity div 4 else
    if FCapacity > 8 then Delta := 16 else
      Delta := 4;
  SetCapacity(FCapacity + Delta);
end;

function TPieComboBoxStrings.IndexOf(const S: string): Integer;
begin
  if not Sorted then Result := inherited IndexOf(S) else
    if not Find(S, Result) then Result := -1;
end;

procedure TPieComboBoxStrings.Insert(Index: Integer; const S: string);
begin
  if Sorted then Error(SSortedListError, 0);
  if (Index < 0) or (Index > FCount) then Error(SListIndexError, Index);
  InsertItem(Index, S);
end;

procedure TPieComboBoxStrings.InsertItem(Index: Integer; const S: string);
VAR
  I: Integer;
begin
  Changing;
  I := FComboBox.ItemIndex;
  if FCount = FCapacity then Grow;
  if Index < FCount then System.Move(FList^[Index],
    FList^[Index+1], (FCount-Index)*SizeOf(TPieComboBoxItem));
 {!!!} Initialize(FList^[Index]);
  with FList^[Index] do begin
    FString := S;
    FPieComboBoxEintrag := TPieComboBoxEintrag.Create;
    FPieComboBoxEintrag.FBitmap := TBitmap.Create;;
    FPieComboBoxEintrag.FColor := TPieComboBox(FComboBox).Color;
    if FPieComboBoxEintrag.FColor < 0 THEN FPieComboBoxEintrag.FColor := $00FFFFFF;
    FPieComboBoxEintrag.FFont := TFont.Create;
    FPieComboBoxEintrag.FFont.Assign(TPieComboBox(FComboBox).Font);
    FPieComboBoxEintrag.FHelpContext := TPieComboBox(FComboBox).HelpContext;
    FPieComboBoxEintrag.FHint := TPieComboBox(FComboBox).Hint;
  end;
  Inc(FCount);
{!!} if SendMessage(FComboBox.Handle, CB_INSERTSTRING, Index, Longint(PChar(S))) < 0 then
       raise EOutOfResources.Create(SInsertLineError);
  IF I<FCount THEN FComboBox.ItemIndex := I;
  Changed;
end;

procedure TPieComboBoxStrings.Put(Index: Integer; const S: string);
var
  I: Integer;
begin
  if Sorted then Error(SSortedListError, 0);
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;
  {!!}I := FComboBox.ItemIndex;
  FList^[Index].FString := S;
  {!!} SendMessage(FComboBox.Handle, CB_DELETESTRING, Index, 0);
  {!!} SendMessage(FComboBox.Handle, CB_INSERTSTRING, Index, Longint(PChar(S)));
  {!!}FComboBox.ItemIndex := I;
  Changed;
end;

procedure TPieComboBoxStrings.PutPieComboBoxEintrag(Index: Integer; APieComboBoxEintrag: TPieComboBoxEintrag);
VAR
  S: string;
  Neu_berechnen: Boolean;
  AHeight: Integer;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;

  Neu_Berechnen :=
    (FList^[Index].FPieComboBoxEintrag.FFont.Height <> APieComboBoxEintrag.FFont.Height) OR
    (FList^[Index].FPieComboBoxEintrag.FBitmap.Height <> APieComboBoxEintrag.FBitmap.Height);

  IF Neu_berechnen THEN BEGIN
    BeginUpdate;
    S := FList^[Index].FString;
    Delete(Index);
    Insert(Index, S);
    FList^[Index].FPieComboBoxEintrag := APieComboBoxEintrag;
    FComboBox.MeasureItem(Index, AHeight);
    SendMessage(FComboBox.Handle, CB_SETITEMHEIGHT, Index, AHeight);
    EndUpdate;
  END
  ELSE FList^[Index].FPieComboBoxEintrag := APieComboBoxEintrag;

  Changed;
end;

procedure TPieComboBoxStrings.QuickSort(L, R: Integer);
var
  I, J: Integer;
  P: string;
begin
  repeat
    I := L;
    J := R;
    P := FList^[(L + R) shr 1].FString;
    repeat
      while AnsiCompareText(FList^[I].FString, P) < 0 do Inc(I);
      while AnsiCompareText(FList^[J].FString, P) > 0 do Dec(J);
      if I <= J then
      begin
        ExchangeItems(I, J);
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort(L, J);
    L := I;
  until I >= R;
end;

procedure TPieComboBoxStrings.ReadBitmapData(Stream: TStream);
VAR
  I: Integer;
  Buffer: char;
begin
  BeginUpdate;
{  TPieComboBox(FComboBox).Style := csOwnerDrawFixed;}
  try
    for I := 0 to Count - 1 do begin
      Stream.ReadBuffer(Buffer, 1);
      IF Buffer = 'Y' THEN
        PieComboBoxEintraege[I].FBitmap.LoadFromStream(Stream);
    end;
  finally
{    TPieComboBox(FComboBox).Style := csOwnerDrawVariable;}
    EndUpdate;
  end;
end;

procedure TPieComboBoxStrings.ReadData(Reader: TReader);
VAR
  E: TPieComboBoxEintrag;
  I: Integer;
begin
  Reader.ReadListBegin;
  BeginUpdate;
{  TPieComboBox(FComboBox).Style := csOwnerDrawFixed;}
  try
    Clear;
    while not Reader.EndOfList do begin
      I := Add(Reader.ReadString);
      E := PieComboBoxEintraege[I];
      E.FColor := Reader.ReadInteger;
      E.FHelpContext := Reader.ReadInteger;
      E.FHint := Reader.ReadString;
      E.FFont.Name := Reader.ReadString;
      E.FFont.CharSet := Reader.ReadInteger;
      E.FFont.Color := Reader.ReadInteger;
      E.FFont.Height := Reader.ReadInteger;
      E.FFont.Pitch := TFontPitch(Reader.ReadInteger);
      E.FFont.Style := [];
      IF Reader.ReadBoolean THEN E.FFont.Style := E.FFont.Style + [fsBold];
      IF Reader.ReadBoolean THEN E.FFont.Style := E.FFont.Style + [fsItalic];
      IF Reader.ReadBoolean THEN E.FFont.Style := E.FFont.Style + [fsUnderline];
      IF Reader.ReadBoolean THEN E.FFont.Style := E.FFont.Style + [fsStrikeOut];
      PieComboBoxEintraege[I] := E;
    END;
  finally
{    TPieComboBox(FComboBox).Style := csOwnerDrawVariable;}
    EndUpdate;
  end;
  Reader.ReadListEnd;
end;

procedure TPieComboBoxStrings.SetCapacity(NewCapacity: Integer);
begin
  ReallocMem(FList, NewCapacity * SizeOf(TPieComboBoxItem));
  FCapacity := NewCapacity;
end;

procedure TPieComboBoxStrings.SetBitmap(Index: Integer; ABitmap: TBitmap);
VAR
  Neu_berechnen: Boolean;
  AHeight: Integer;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;

  Neu_Berechnen :=
    (FList^[Index].FPieComboBoxEintrag.FBitmap.Height <> ABitmap.Height);

  IF Neu_berechnen THEN BEGIN
    BeginUpdate;
    FList^[Index].FPieComboBoxEintrag.FBitmap.Assign(ABitmap);
    FComboBox.MeasureItem(Index, AHeight);
    SendMessage(FComboBox.Handle, CB_SETITEMHEIGHT, Index, AHeight);
    FComboBox.Repaint;
    EndUpdate;
  END
  ELSE FList^[Index].FPieComboBoxEintrag.FBitmap.Assign(ABitmap);

  Changed;
end;

procedure TPieComboBoxStrings.SetColor(Index: Integer; AColor: TColor);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;
  FList^[Index].FPieComboBoxEintrag.FColor := AColor;
  FComboBox.Repaint;
  Changed;
end;

procedure TPieComboBoxStrings.SetFont(Index: Integer; AFont: TFont);
VAR
  Neu_berechnen: Boolean;
  AHeight: Integer;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;

  Neu_Berechnen := (FList^[Index].FPieComboBoxEintrag.FFont.Height <> AFont.Height);

  IF Neu_berechnen THEN BEGIN
    BeginUpdate;
    FList^[Index].FPieComboBoxEintrag.FFont.Assign(AFont);
    FComboBox.MeasureItem(Index, AHeight);
    SendMessage(FComboBox.Handle, CB_SETITEMHEIGHT, Index, AHeight);
    FComboBox.Repaint;
    EndUpdate;
  END
  ELSE FList^[Index].FPieComboBoxEintrag.FFont.Assign(AFont);

  Changed;
end;

procedure TPieComboBoxStrings.SetHelpContext(Index: Integer; AHelpContext: THelpContext);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;

  FList^[Index].FPieComboBoxEintrag.FHelpContext := AHelpContext;

  Changed;
end;

procedure TPieComboBoxStrings.SetHint(Index: Integer; AHint: string);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Changing;

  FList^[Index].FPieComboBoxEintrag.FHint := AHint;

  Changed;
end;

procedure TPieComboBoxStrings.SetSorted(Value: Boolean);
begin
  if FSorted <> Value then
  begin
    if Value then Sort;
    FSorted := Value;
  end;
end;

procedure TPieComboBoxStrings.SetUpdateState(Updating: Boolean);
begin
  if Updating then Changing else Changed;

(*{!!} SendMessage(FComboBox.Handle, WM_SETREDRAW, Ord(not Updating), 0);
{!!} if not Updating then FComboBox.Refresh;*)
end;

procedure TPieComboBoxStrings.Sort;
begin
  if not Sorted and (FCount > 1) then
  begin
    Changing;
    QuickSort(0, FCount - 1);
    Changed;
  end;
end;

procedure TPieComboBoxStrings.WriteBitmapData(Stream: TStream);
var
  I: Integer;
  E: TPieComboBoxEintrag;
  Buffer: char;
begin
  for I := 0 to Count - 1 do begin
    E := PieComboBoxEintraege[I];
    IF E.FBitmap.Empty THEN Buffer := 'N' ELSE Buffer := 'Y';
    Stream.WriteBuffer(Buffer, 1);
    E.FBitmap.SaveToStream(Stream);
  end;
end;

procedure TPieComboBoxStrings.WriteData(Writer: TWriter);
var
  I: Integer;
  E: TPieComboBoxEintrag;
begin
  Writer.WriteListBegin;
  for I := 0 to Count - 1 do begin
    Writer.WriteString(Get(I));
    E := PieComboBoxEintraege[I];
    Writer.WriteInteger(E.FColor);
    Writer.WriteInteger(E.FHelpContext);
    Writer.WriteString(E.FHint);
    Writer.WriteString(E.FFont.Name);
    Writer.WriteInteger(E.FFont.CharSet);
    Writer.WriteInteger(E.FFont.Color);
    Writer.WriteInteger(E.FFont.Height);
    Writer.WriteInteger(Integer(E.FFont.Pitch));
    Writer.WriteBoolean(fsBold IN E.FFont.Style);
    Writer.WriteBoolean(fsItalic IN E.FFont.Style);
    Writer.WriteBoolean(fsUnderline IN E.FFont.Style);
    Writer.WriteBoolean(fsStrikeOut IN E.FFont.Style);
  end;
  Writer.WriteListEnd;
end;
{******************************************************}
{******************************************************}
{******************************************************}
function TPieComboBoxEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TPieComboBoxEditor.GetVerb(Index: Integer): string;
begin
  CASE Index OF
  0: Result := '&Edit PieComboBox';
  END;
end;

procedure TPieComboBoxEditor.ExecuteVerb(Index: Integer);
var
  D : TPieComboBoxItemsfenster;
begin
  CASE Index OF
  0: BEGIN
     D := TPieComboBoxItemsfenster.Create(Application);
     try
       D.Caption := Component.Owner.Name + '.' +
                    Component.Name + ' - ' + D.Caption;

       D.ComboBox.CopyProp(D.ComboBox, Component as TPieComboBox);
       if D.ShowModal = mrOK then
         D.ComboBox.CopyProp(Component as TPieComboBox, D.ComboBox);
     finally
       D.Free;
     end;
     END;
  END;
end;

procedure TPieComboBox.Edit_aufrufen;
var
  D : TPieComboBoxItemsfenster;
begin
  D := TPieComboBoxItemsfenster.Create(Application);
  try
    CopyProp(D.ComboBox, self);
    if D.ShowModal = mrOK then
      CopyProp(self, D.ComboBox);
  finally
    D.Free;
  end;
end;

procedure TPieComboBox.CopyProp(Dest, Source: TPieComboBox);
BEGIN
  Dest.Items.Assign(Source.Items);
  Dest.Font.Assign(Source.Font);
  Dest.HeaderSections.Assign(Source.HeaderSections);
  Dest.HeaderFont.Assign(Source.HeaderFont);
  Dest.BackGround.Assign(Source.BackGround);
  Dest.BackGroundStyle := Source.BackGroundStyle;
  Dest.LineByTabStops  := Source.LineByTabStops;
  Dest.BitmapStyle     := Source.BitmapStyle;
  Dest.ItemHeightStyle := Source.ItemHeightStyle;
  Dest.AnyColors       := Source.AnyColors;
  Dest.AnyFonts        := Source.AnyFonts;
  Dest.Color           := Source.Color;
  Dest.HelpContext     := Source.HelpContext;
  Dest.Hint            := Source.Hint;
  Dest.HeaderControl   := Source.HeaderControl;
  Dest.HeaderStyle     := Source.HeaderStyle;
  Dest.BoldStyle       := Source.BoldStyle;
END;

{******************************************************}
{******************************************************}
{******************************************************}
{ TPieComboBox }

constructor TPieComboBox.Create(AOwner: TComponent);
const
  ComboBoxStyle = [csCaptureMouse, csSetCaption, csDoubleClicks,
    csFixedHeight, csReflector];
begin
  inherited Create(AOwner);
  if NewStyleControls then
    ControlStyle := ComboBoxStyle else
    ControlStyle := ComboBoxStyle + [csFramed];
  Width := 145;
  Height := 25;
  TabStop := True;
  ParentColor := False;
  FItems := TPieComboBoxStrings.Create(Self);
  FCanvas := TControlCanvas.Create;
  FBackground := TPicture.Create;
  FBackground.OnChange := BackgroundChange;
  FBackGroundStyle := bgscNone;
  Header := THeaderControl.Create(Self);
  Header.Visible := FALSE;
  Header.Align := alNone;
  Header.OnSectionResize := HeaderSectionResize;
  FItemHeight := 16;
  FEditInstance := MakeObjectInstance(EditWndProc);
  FListInstance := MakeObjectInstance(ListWndProc);
  FDropDownCount := 8;
  FAnyColors := TRUE;
  FAnyFonts := TRUE;
  FItemHeightStyle := ihcFix;
  FBitmapStyle := bscLeft;
  FBoldStyle := FALSE;
  FStyle := csSimple;
  FLineByTabStops := FALSE;
  FVersion := '1.1';
end;

destructor TPieComboBox.Destroy;
begin
  if HandleAllocated then DestroyWindowHandle;
  FreeObjectInstance(FListInstance);
  FreeObjectInstance(FEditInstance);
  FCanvas.Free;
  FItems.Free;
  FSaveItems.Free;
  FBackground.Free;
  inherited Destroy;
end;

procedure TPieComboBox.Clear;
begin
  SetTextBuf('');
  FItems.Clear;
end;

procedure TPieComboBox.SelectAll;
begin
  SendMessage(Handle, CB_SETEDITSEL, 0, (-1 SHL 16));
end;

function TPieComboBox.GetDroppedDown: Boolean;
begin
  Result := LongBool(SendMessage(Handle, CB_GETDROPPEDSTATE, 0, 0));
end;

procedure TPieComboBox.SetDroppedDown(Value: Boolean);
begin
  SendMessage(Handle, CB_SHOWDROPDOWN, Longint(Value), 0);
end;

function TPieComboBox.GetItemIndex: Integer;
begin
  Result := SendMessage(Handle, CB_GETCURSEL, 0, 0);
end;

procedure TPieComboBox.SetItemIndex(Value: Integer);
begin
  SendMessage(Handle, CB_SETCURSEL, Value, 0);
end;

function TPieComboBox.GetSelStart: Integer;
begin
  SendMessage(Handle, CB_GETEDITSEL, Longint(@Result), 0);
end;

procedure TPieComboBox.SetSelStart(Value: Integer);
var
  Selection: TSelection;
begin
  Selection.StartPos := Value;
  Selection.EndPos := Value;
  SendMessage(Handle, CB_SETEDITSEL, 0, MakeLParam(Selection.StartPos,
    Selection.EndPos));
end;

function TPieComboBox.GetSelLength: Integer;
var
  Selection: TSelection;
begin
  SendMessage(Handle, CB_GETEDITSEL, Longint(@Selection.StartPos),
    Longint(@Selection.EndPos));
  Result := Selection.EndPos - Selection.StartPos;
end;

procedure TPieComboBox.SetSelLength(Value: Integer);
var
  Selection: TSelection;
begin
  SendMessage(Handle, CB_GETEDITSEL, Longint(@Selection.StartPos),
    Longint(@Selection.EndPos));
  Selection.EndPos := Selection.StartPos + Value;
  SendMessage(Handle, CB_SETEDITSEL, 0, MakeLParam(Selection.StartPos,
    Selection.EndPos));
end;

function TPieComboBox.GetSelText: string;
begin
  Result := '';
  if FStyle < csDropDownList then
    Result := Copy(Text, GetSelStart + 1, GetSelLength);
end;

procedure TPieComboBox.SetSelText(const Value: string);
begin
  if FStyle < csDropDownList then
  begin
    HandleNeeded;
    SendMessage(FEditHandle, EM_REPLACESEL, 0, Longint(PChar(Value)));
  end;
end;

procedure TPieComboBox.SetMaxLength(Value: Integer);
begin
  if Value < 0 then Value := 0;
  if FMaxLength <> Value then
  begin
    FMaxLength := Value;
    if HandleAllocated then SendMessage(Handle, CB_LIMITTEXT, Value, 0);
  end;
end;

procedure TPieComboBox.SetSorted(Value: Boolean);
begin
  if FSorted <> Value then
  begin
    FSorted := Value;
    RecreateWnd;
  end;
end;

procedure TPieComboBox.SetStyle(Value: TComboBoxStyle);
VAR
  Index: Integer;
begin
  if FStyle <> Value then
  begin
    FStyle := Value;
    Index := Perform(CB_GETCURSEL, 0, 0);
    if Value = csSimple then
      ControlStyle := ControlStyle - [csFixedHeight] else
      ControlStyle := ControlStyle + [csFixedHeight];
    RecreateWnd;
    Perform(CB_SETCURSEL, Index, 0);
  end;
end;

function TPieComboBox.GetItemHeight: Integer;
begin
  if FStyle in [csOwnerDrawFixed, csOwnerDrawVariable] then
    Result := FItemHeight else
    Result := Perform(CB_GETITEMHEIGHT, -1, 0);
end;

procedure TPieComboBox.SetItemHeight(Value: Integer);
begin
  if Value > 0 then BEGIN
    FItemHeight := Value;
    Perform(CB_SETITEMHEIGHT, -1, FItemHeight);
  END;
end;

procedure TPieComboBox.SetItems(Value: TPieComboBoxStrings);
begin
  Items.Assign(Value);
end;

procedure TPieComboBox.CreateParams(var Params: TCreateParams);
const
  ComboBoxStyles: array[TComboBoxStyle] of Integer = (
    CBS_DROPDOWN or CBS_OWNERDRAWVARIABLE,
    CBS_SIMPLE or CBS_OWNERDRAWVARIABLE,
    CBS_DROPDOWNLIST or CBS_OWNERDRAWVARIABLE,
    CBS_DROPDOWNLIST or CBS_OWNERDRAWFIXED,
    CBS_DROPDOWNLIST or CBS_OWNERDRAWVARIABLE);
  Sorts: array[Boolean] of Integer = (0, CBS_SORT);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'COMBOBOX');
  with Params do
  begin
    Style := Style or (WS_VSCROLL or CBS_HASSTRINGS or CBS_AUTOHSCROLL) or
      word(ComboBoxStyles[FStyle]) or word(Sorts[FSorted]);
    if NewStyleControls and Ctl3D then
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    if HeaderControl then Style := Style or LBS_USETABSTOPS;
  end;
end;

procedure TPieComboBox.CreateWnd;
var
  ChildHandle: THandle;
begin
  inherited CreateWnd;
  IF Header.Parent = NIL THEN BEGIN
    Header.Parent := Parent;
    Header.Name := Name + 'Header';
  END;
  SendMessage(Handle, CB_LIMITTEXT, FMaxLength, 0);
  if FSaveItems <> nil then
  begin
    FItems.Assign(FSaveItems);
    FSaveItems.Free;
    FSaveItems := nil;
  end;
  FEditHandle := 0;
  FListHandle := 0;
  if FStyle in [csDropDown, csSimple] then
  begin
    ChildHandle := GetWindow(Handle, GW_CHILD);
    if ChildHandle <> 0 then
    begin
      if FStyle = csSimple then
      begin
        FListHandle := ChildHandle;
        FDefListProc := Pointer(GetWindowLong(FListHandle, GWL_WNDPROC));
        SetWindowLong(FListHandle, GWL_WNDPROC, Longint(FListInstance));
        ChildHandle := GetWindow(ChildHandle, GW_HWNDNEXT);
      end;
      FEditHandle := ChildHandle;
      FDefEditProc := Pointer(GetWindowLong(FEditHandle, GWL_WNDPROC));
      SetWindowLong(FEditHandle, GWL_WNDPROC, Longint(FEditInstance));
    end;
  end;
  if NewStyleControls and (FEditHandle <> 0) then
    SendMessage(FEditHandle, EM_SETMARGINS, EC_LEFTMARGIN or EC_RIGHTMARGIN, 0);
end;

procedure TPieComboBox.DestroyWnd;
begin
  if FItems.Count > 0 then
  begin
    FSaveItems := TPieComboBoxStrings.Create(Self);
    FSaveItems.Assign(FItems);
  end;
  inherited DestroyWnd;
end;

PROCEDURE TPieComboBox.Loaded;
BEGIN
  IF Header.Parent = NIL THEN BEGIN
    Header.Parent := Parent;
    Header.Name := Name+'Header';
  END;
  inherited Loaded;
END;

procedure TPieComboBox.SetBackground(Value: TPicture);
BEGIN
  FBackground.Assign(Value);
  Invalidate;
END;

procedure TPieComboBox.BackgroundChange(Sender: TObject);
BEGIN
  Invalidate;
END;

procedure TPieComboBox.SetBackGroundStyle(Value: TBackGroundStyle);
BEGIN
  IF Value <> FBackGroundStyle THEN BEGIN
    FBackGroundStyle := Value;
    Invalidate;
  END;
END;

procedure TPieComboBox.WMMove(var Message: TWMMove);
begin
  inherited;
  IF HeaderControl
    THEN Header.SetBounds(Left, Top-Header.Height, Width, Header.Height);
end;

procedure TPieComboBox.WMSize(var Message: TWMSize);
begin
  inherited;
  IF HeaderControl
    THEN Header.SetBounds(Left, Top-Header.Height, Width, Header.Height);
end;

procedure TPieComboBox.WMCreate(var Message: TWMCreate);
begin
  inherited;
  SetWindowText(Handle, WindowText);
end;

procedure TPieComboBox.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  DefaultHandler(Message);
end;

procedure TPieComboBox.WMPaint(var Message: TWmPaint);
BEGIN
  inherited;
END;

procedure TPieComboBox.PaintBackground(R: TRect; Edit_malen: Boolean);
VAR
  CR: TRect;
  B, H, I, J: Integer;
  P : TPicture;
  Pos1: TRect; {Position des gesamten Pattern im Canvas}
  Pos2: TRect; {Position des darzustellenden Teilpattern im Pattern}
  Malen: Boolean;
  EditDC: HDC;
BEGIN     {TCanvas, Pattern}
  IF Assigned(FBackGround) AND Assigned(FBackGround.Graphic) THEN BEGIN
    P := TPicture.Create;
    TRY
      P.Assign(FBackGround);
      IF Edit_malen
        THEN windows.GetClientRect(FEditHandle, CR)
      ELSE BEGIN
        CR := ClientRect;      {gibt Mae von Edit zurck}
//        CR.Bottom := R.Bottom; {wichtig! Hhe fr Liste anpassen}
      END;
      H := P.Graphic.Height;
      B := P.Graphic.Width;
      IF (H>0) AND (B>0) THEN BEGIN
        CASE FBackGroundStyle OF
{------------------------------------------------------}
        bgscCenter:
          BEGIN
            Pos1 := Rect((CR.Right-CR.Left-B) DIV 2,
                         (CR.Bottom-CR.Top-H) DIV 2,
                         (CR.Right-CR.Left+B) DIV 2,
                         (CR.Bottom-CR.Top+H) DIV 2);

            Malen := (Pos1.Left < R.Right) AND (Pos1.Right > R.Left) AND
                     (Pos1.Top < R.Bottom) AND (Pos1.Bottom > R.Top);

            IF Malen THEN BEGIN
              Pos2 := Rect(R.Left-Pos1.Left, R.Top-Pos1.Top, R.Right-Pos1.Left, R.Bottom-Pos1.Top);
              IF Pos2.Left   < 0 THEN Pos2.Left   := 0;
              IF Pos2.Top    < 0 THEN Pos2.Top    := 0;
              IF Pos2.Right  > B THEN Pos2.Right  := B;
              IF Pos2.Bottom > H THEN Pos2.Bottom := H;

              IF (Pos2.Left > 0) OR (Pos2.Top > 0) THEN
              BitBlt(P.Bitmap.Canvas.Handle, 0, 0,
                    (Pos2.Right-Pos2.Left),
                    (Pos2.Bottom-Pos2.Top),
                     P.Bitmap.Canvas.Handle, Pos2.Left, Pos2.Top,
                     srcCopy);

              P.Graphic.Width := Pos2.Right-Pos2.Left;
              P.Graphic.Height := Pos2.Bottom-Pos2.Top;
              Canvas.Draw(Pos1.Left + Pos2.Left, Pos1.Top  + Pos2.Top, P.Graphic);
            END;
          END;
{------------------------------------------------------}
        bgscStretch:
          BEGIN
            Pos1 := CR;

            Pos2 := Rect(R.Left * B DIV (Pos1.Right-Pos1.Left),
                         R.Top * H DIV (Pos1.Bottom-Pos1.Top),
                         R.Right * B DIV (Pos1.Right-Pos1.Left),
                         R.Bottom * H DIV (Pos1.Bottom-Pos1.Top));

            IF (Pos2.Left > 0) OR (Pos2.Top > 0) THEN
            BitBlt(P.Bitmap.Canvas.Handle, 0, 0,
                  (Pos2.Right-Pos2.Left),
                  (Pos2.Bottom-Pos2.Top),
                   P.Bitmap.Canvas.Handle, Pos2.Left, Pos2.Top,
                   srcCopy);

            P.Graphic.Width := Pos2.Right-Pos2.Left;
            P.Graphic.Height := Pos2.Bottom-Pos2.Top;
            Canvas.StretchDraw(R, P.Graphic);
          END;
{------------------------------------------------------}
        bgscPattern:
          BEGIN
            FOR I := 0 TO ((CR.Right - CR.Left) DIV B) DO
            FOR J := 0 TO ((CR.Bottom - CR.Top) DIV H) DO BEGIN
              Pos1 := Rect(I*B, J*H, (I+1)*B, (J+1)*H);
              Malen := (Pos1.Left < R.Right) AND (Pos1.Right > R.Left) AND
                       (Pos1.Top < R.Bottom) AND (Pos1.Bottom > R.Top);

              IF Malen THEN BEGIN
                Pos2 := Rect(R.Left-I*B, R.Top-J*H, R.Right-I*B, R.Bottom-J*H);
                IF Pos2.Left   < 0 THEN Pos2.Left   := 0;
                IF Pos2.Top    < 0 THEN Pos2.Top    := 0;
                IF Pos2.Right  > B THEN Pos2.Right  := B;
                IF Pos2.Bottom > H THEN Pos2.Bottom := H;

                IF (Pos2.Left > 0) OR (Pos2.Top > 0) THEN
                BitBlt(P.Bitmap.Canvas.Handle, 0, 0,
                      (Pos2.Right-Pos2.Left),
                      (Pos2.Bottom-Pos2.Top),
                       P.Bitmap.Canvas.Handle, Pos2.Left, Pos2.Top,
                       srcCopy);

                P.Graphic.Width := Pos2.Right-Pos2.Left;
                P.Graphic.Height := Pos2.Bottom-Pos2.Top;
                IF Edit_malen THEN BEGIN
                  EditDC := GetDC(EditHandle);
                  TRY
                    BitBlt(EditDC, Pos1.Left + Pos2.Left, Pos1.Top  + Pos2.Top,
                           P.Graphic.Width,
                           P.Graphic.Height,
                           P.Bitmap.Canvas.Handle,
                           0, 0, srcCopy);
                  FINALLY
                    ReleaseDC(EditHandle, EditDC);
                  END;
                END
                ELSE Canvas.Draw(Pos1.Left + Pos2.Left, Pos1.Top  + Pos2.Top, P.Graphic);
                P.Assign(FBackGround);
              END;
            END;
          END;
{------------------------------------------------------}
        END;  {CASE FBackGroundStyle OF ...}
      END;
    FINALLY
      P.Free;
    END;
  END;
END;

procedure TPieComboBox.WMDrawItem(var Message: TWMDrawItem);
begin
  DefaultHandler(Message);
end;

procedure TPieComboBox.WMMeasureItem(var Message: TWMMeasureItem);
begin
  DefaultHandler(Message);
end;

procedure TPieComboBox.WMDeleteItem(var Message: TWMDeleteItem);
begin
  DefaultHandler(Message);
end;

procedure TPieComboBox.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  if DroppedDown then Message.Result := Message.Result or DLGC_WANTALLKEYS;
end;

procedure TPieComboBox.CMCancelMode(var Message: TCMCancelMode);
begin
  if Message.Sender <> Self then Perform(CB_SHOWDROPDOWN, 0, 0);
end;

procedure TPieComboBox.CMCtl3DChanged(var Message: TMessage);
begin
  if NewStyleControls then RecreateWnd;
  inherited;
end;

procedure TPieComboBox.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if not NewStyleControls and (Style < csDropDownList) then Invalidate;
end;

procedure TPieComboBox.EditWndProc(var Message: TMessage);
var
  P: TPoint;
  Form: TCustomForm;
begin
  if Message.Msg = WM_SYSCOMMAND then
  begin
    WndProc(Message);
    Exit;
  end
  else if (Message.Msg >= WM_KEYFIRST) and (Message.Msg <= WM_KEYLAST) then
  begin
    Form := GetParentForm(Self);
    if (Form <> nil) and Form.WantChildKey(Self, Message) then Exit;
  end;
  IF (Message.Msg = wm_SetText) THEN;
  ComboWndProc(Message, FEditHandle, FDefEditProc); {513, 8, wm_Paint}
  case Message.Msg of
    WM_LBUTTONDOWN, WM_LBUTTONDBLCLK:
      begin
        if DragMode = dmAutomatic then
        begin
          GetCursorPos(P);
          P := ScreenToClient(P);
          SendMessage(FEditHandle, WM_LBUTTONUP, 0,Longint(PointToSmallPoint(P)));
          BeginDrag(False);
        end;
      end;
    WM_SETFONT:
      if NewStyleControls then
        SendMessage(FEditHandle, EM_SETMARGINS, EC_LEFTMARGIN or EC_RIGHTMARGIN, 0);
  end;
end;

procedure TPieComboBox.ListWndProc(var Message: TMessage);
begin
  ComboWndProc(Message, FListHandle, FDefListProc);
end;

procedure TPieComboBox.ComboWndProc(var Message: TMessage; ComboWnd: HWnd;
  ComboProc: Pointer);
var
  Point: TPoint;
  Form: TCustomForm;
  ST: TShiftState;
begin
  try
    with Message do
    begin
      case Msg of
        WM_SETFOCUS:
          begin
            Form := GetParentForm(Self);
            if (Form <> nil) and not Form.SetFocusedControl(Self) then Exit;
          end;
        WM_KILLFOCUS:
          if csFocusing in ControlState then Exit;
        WM_KEYDOWN, WM_SYSKEYDOWN:
          if (ComboWnd <> FListHandle) and DoKeyDown(TWMKey(Message)) then
            Exit;
        WM_CHAR:
          begin
            if DoKeyPress(TWMKey(Message)) then Exit;
            if ((TWMKey(Message).CharCode = VK_RETURN) or
              (TWMKey(Message).CharCode = VK_ESCAPE)) and DroppedDown then
            begin
              DroppedDown := False;
              Exit;
            end;
          end;
        WM_KEYUP, WM_SYSKEYUP:
          if DoKeyUp(TWMKey(Message)) then Exit;
        WM_MOUSEMOVE: IF NOT(Dragging) THEN UpdateHint(-1);
        WM_LBUTTONUP: IF assigned(OnDragDrop) THEN OnDragDrop(Self, Self, LoWord(LParam), HiWord(LParam));
        WM_LBUTTONDOWN:
          IF assigned(OnMouseDown) THEN BEGIN
            ST := [];
            IF wParam = MK_CONTROL THEN ST := ST + [ssCtrl];
            IF wParam = MK_LBUTTON THEN ST := ST + [ssLeft];
            IF wParam = MK_MBUTTON THEN ST := ST + [ssMiddle];
            IF wParam = MK_RBUTTON THEN ST := ST + [ssRight];
            IF wParam = MK_SHIFT   THEN ST := ST + [ssShift];
            OnMouseDown(self, mbLeft, ST, LoWord(lparam), HiWord(lparam));
          END;
        WM_RBUTTONUP:
          if HasPopup(Self) then
          begin
            with TWMRButtonUp(Message) do
            begin
              Point.X := Pos.X;
              Point.Y := Pos.Y;
              MapWindowPoints(ComboWnd, Handle, Point, 1);
              Pos.X := Point.X;
              Pos.Y := Point.Y;
            end;
            WndProc(Message);
            Exit;
          end;
        WM_GETDLGCODE:
          if DroppedDown then
          begin
            Result := DLGC_WANTALLKEYS;
            Exit;
          end;
        WM_NCHITTEST:
          if csDesigning in ComponentState then
          begin
            Result := HTTRANSPARENT;
            Exit;
          end;
        CN_KEYDOWN, CN_CHAR, CN_SYSKEYDOWN, CN_SYSCHAR:
          begin
            WndProc(Message);
            Exit;
          end;
      end;
      Result := CallWindowProc(ComboProc, ComboWnd, Msg, WParam, LParam);
      if (Msg = WM_LBUTTONDBLCLK) and (csDoubleClicks in ControlStyle) then
        DblClick;
    end;
  except
    Application.HandleException(Self);
  end;
end;

procedure TPieComboBox.WndProc(var Message: TMessage);
begin
    {for auto drag mode, let listbox handle itself, instead of TControl}
  if not (csDesigning in ComponentState) and
     ((Message.Msg = WM_LBUTTONDOWN) or (Message.Msg = WM_LBUTTONDBLCLK)) and
     not Dragging then
  begin
    if DragMode = dmAutomatic then
    begin
      if IsControlMouseMsg(TWMMouse(Message)) then
        Exit;
      ControlState := ControlState + [csLButtonDown];
      Dispatch(Message);  {overrides TControl's BeginDrag}
      Exit;
    end;
  end;
  with Message do
    case Msg of
      WM_CTLCOLORMSGBOX..WM_CTLCOLORSTATIC:
        begin
          SetTextColor(WParam, ColorToRGB(Font.Color));
          SetBkColor(WParam, ColorToRGB(Brush.Color));
          Result := Brush.Handle;{$R+}
          Exit;
        end;
      CN_CTLCOLORMSGBOX..CN_CTLCOLORSTATIC:
        if not NewStyleControls and (Style < csDropDownList) then
        begin
          Result := Parent.Brush.Handle;
          Exit;
        end;
      WM_CHAR:
        begin
          if DoKeyPress(TWMKey(Message)) then Exit;
          if ((TWMKey(Message).CharCode = VK_RETURN) or
            (TWMKey(Message).CharCode = VK_ESCAPE)) and DroppedDown then
          begin
            DroppedDown := False;
            Exit;
          end;
        end;
    end;
  inherited WndProc(Message);
end;

procedure TPieComboBox.AdjustDropDown;
var
  ItemCount: Integer;
begin
  ItemCount := FItems.Count;
  if ItemCount > DropDownCount then ItemCount := DropDownCount;
  if ItemCount < 1 then ItemCount := 1;
  SetWindowPos(Handle, 0, 0, 0, Width, ItemHeight * ItemCount +
    Height + 2, SWP_NOMOVE + SWP_NOZORDER + SWP_NOACTIVATE + SWP_NOREDRAW +
    SWP_HIDEWINDOW);
  SetWindowPos(Handle, 0, 0, 0, 0, 0, SWP_NOMOVE + SWP_NOSIZE +
    SWP_NOZORDER + SWP_NOACTIVATE + SWP_NOREDRAW + SWP_SHOWWINDOW);
end;

procedure TPieComboBox.CNCommand(var Message: TWMCommand);
begin
  case Message.NotifyCode of
    CBN_DBLCLK:
      DblClick;
    CBN_EDITCHANGE:
      Change;
    CBN_DROPDOWN:
      begin
        FFocusChanged := False;
        DropDown;
        AdjustDropDown;
        if FFocusChanged then
        begin
          PostMessage(Handle, WM_CANCELMODE, 0, 0);
          if not FIsFocused then PostMessage(Handle, CB_SHOWDROPDOWN, 0, 0);
        end;
      end;
    CBN_SELCHANGE:
      begin
        IF ItemIndex > -1 THEN BEGIN
          Text := Items[ItemIndex];
          Click;
          Change;
        END;
      end;
    CBN_SETFOCUS:
      begin
        FIsFocused := True;
        FFocusChanged := True;
        SetIme;
      end;
    CBN_KILLFOCUS:
      begin
        FIsFocused := False;
        FFocusChanged := True;
        ResetIme;
      end;
  end;
end;

procedure TPieComboBox.Change;
begin
  inherited Changed;
  if Assigned(FOnChange) then FOnChange(Self);
  SetItemProperties;
end;

procedure TPieComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
VAR
  C: TColor;
begin
  C := Font.Color;
  case FStyle of
    csDropDown, csSimple: DrawBoldStyle(Index, Rect, State);
    csDropDownList: DrawBoldStyle(Index, Rect, State);
    csOwnerDrawFixed,
    csOwnerDrawVariable: if Assigned(FOnDrawItem) then FOnDrawItem(Self, Index, Rect, State);
  end;
  Font.Color := C;
end;

procedure TPieComboBox.DropDown;
begin
  if Assigned(FOnDropDown) then FOnDropDown(Self);
end;

procedure TPieComboBox.MeasureItem(Index: Integer; var AHeight: Integer);
var
  Metrics: TTextMetric;
  H, B: Integer;
  DC: HDC;
begin
  IF {(FStyle <> csOwnerDraw) AND} (Index >-1) THEN WITH Canvas do begin
    {Standard-hhe}
    DC := GetDC(0);
    TRY
      Canvas.Handle := DC;
      Canvas.Font := Self.Font;
      GetTextMetrics(DC, Metrics);
      IF Metrics.tmHeight > AHeight THEN AHeight := Metrics.tmHeight;
      {abweichende Hhe}
      CASE FItemHeightStyle OF
      ihcFix: ;
      ihcFont  : BEGIN
                IF FAnyFonts THEN Canvas.Font := Items.Fonts[Index]
                             ELSE Canvas.Font := Self.Font;
                GetTextMetrics(Canvas.Handle, Metrics);
                AHeight := Metrics.tmHeight;
                END;
      ihcBitmap: BEGIN
                IF NOT(Items.Bitmaps[Index].Empty) THEN BEGIN
                  H := Items.Bitmaps[Index].Height;
                  B := Items.Bitmaps[Index].Width;
                  IF Width / B < 1 THEN H := round(Width / B * H);
                  AHeight := H;
                END;
                END;
      ihcFontBitmap: BEGIN
                IF FAnyFonts THEN Canvas.Font := Items.Fonts[Index]
                             ELSE Canvas.Font := Self.Font;
                GetTextMetrics(Canvas.Handle, Metrics);
                AHeight := Metrics.tmHeight;
                IF NOT(Items.Bitmaps[Index].Empty) THEN BEGIN
                  H := Items.Bitmaps[Index].Height;
                  B := Items.Bitmaps[Index].Width;
                  IF Width / B < 1 THEN H := round(Width / B * H);
                  IF H > AHeight THEN AHeight := H;
                END;
                END;
      END;
    FINALLY
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    END;
  END ELSE if Assigned(FOnMeasureItem) then FOnMeasureItem(Self, Index, AHeight)
end;

procedure TPieComboBox.CNDrawItem(var Message: TWMDrawItem);
var
  State: TOwnerDrawState;
begin
  with Message.DrawItemStruct^ do
  begin
{    State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);}
    State := TOwnerDrawState(Word(Message.DrawItemStruct^.ItemState));
    FCanvas.Handle := hDC;
    FCanvas.Font := Font;
    FCanvas.Brush := Brush;
    if (ItemID < uint(Items.Count)) THEN BEGIN
      IF (odSelected in State) then  begin
        FCanvas.Brush.Color := clHighlight;
        FCanvas.Font.Color := clHighlightText
      end;
      DrawItem(itemID, rcItem, State);
    END else FCanvas.FillRect(rcItem);
    if odFocused in State then DrawFocusRect(hDC, rcItem);
    FCanvas.Handle := 0;
  end;
end;

procedure TPieComboBox.CNMeasureItem(var Message: TWMMeasureItem);
begin
  with Message.MeasureItemStruct^ do
  begin
    itemHeight := FItemHeight;
    if FStyle = csOwnerDrawVariable then
      MeasureItem(itemID, Integer(itemHeight));
  end;
end;

procedure TPieComboBox.WMLButtonDown(var Message: TWMLButtonDown);
var
  Form: TCustomForm;
begin
  if (DragMode = dmAutomatic) and (Style = csDropDownList) and
      (Message.XPos < (Width - GetSystemMetrics(SM_CXHSCROLL))) then
  begin
    SetFocus;
    BeginDrag(False);
    Exit;
  end;
  inherited;
  if MouseCapture then
  begin
    Form := GetParentForm(Self);
    if (Form <> nil) and (Form.ActiveControl <> Self) then
      MouseCapture := False;
  end;
end;

function TPieComboBox.DrawItemBitmap(Index: integer; ARect: TRect): TRect;
VAR
  R: TRect;
  H, B, M: Integer;
  Fak1: Double;
begin
  R := ARect;
  H := Items.Bitmaps[Index].Height;
  B := Items.Bitmaps[Index].Width;
  IF (H>0) AND (B>0) THEN BEGIN
    Fak1 := (R.Bottom-R.Top);
    Fak1 := Fak1 / H;
    IF FBitmapStyle = bscLeft THEN BEGIN
      R.Bottom  := R.Top + round(H*Fak1);
      R.Right   := R.Left + round(B*Fak1);
    END;
    IF FBitmapStyle = bscRight THEN BEGIN
      R.Bottom  := R.Top + round(H*Fak1);
      R.Left    := R.Right - round(B*Fak1);
    END;
    IF FBitmapStyle = bscCenter THEN BEGIN
      R.Bottom  := R.Top + round(H*Fak1);
      M         := (R.Right-R.Left) DIV 2;
      R.Left    := M - round(B*Fak1) DIV 2;
      R.Right   := M + round(B*Fak1) DIV 2;
    END;
    IF FBitmapStyle = bscStretch THEN BEGIN
    END;
    Canvas.StretchDraw(R, Items.Bitmaps[Index]);
  END ELSE R := Rect(R.Left,R.Top,R.Left,R.Top);
  Result := R;
end;

procedure TPieComboBox.DrawBoldStyle(Index: integer; ARect: TRect; State: TOwnerDrawState);
TYPE
  PInteger = ^Integer;
var  {Art = 0: Normal, 1: BoldStyle}
  ch: array[0..255] of char;
  TabArray, P: PInteger;
  I, TabMerker: integer;
  BitmapRect, LastRect: TRect;
begin
{************** Einstellungen vornehmen *******************}
  with Canvas do begin
    {manually set these colors to override the color change when
    the item is focused}
    IF FAnyColors
      THEN Brush.Color := Items.Colors[Index]
      ELSE Brush.Color := Color;
    IF FAnyFonts
      THEN Pen.Color := Items.Fonts[Index].Color
      ELSE Pen.Color := Self.Font.Color;
    IF FAnyFonts
      THEN Font := Items.Fonts[Index]
      ELSE Font := Self.Font;

    IF (FBackGroundStyle = bgscNone) OR
       (FAnyColors AND (Brush.Color <> Color))
       THEN FillRect(ARect)
       ELSE PaintBackground(ARect, FALSE);
    {Bei letztem Index: Hintergrund bis zum Schlu restaurieren}
    IF (FBackGroundStyle <> bgscNone) AND (Index = Items.Count-1) THEN BEGIN
      LastRect := ARect;
      LastRect.Top := LastRect.Bottom;
      LastRect.Bottom := ClientRect.Bottom;
{      PaintBackground(LastRect, FALSE);}
    END;

    BitmapRect := Rect(0,0,0,0);
    BitmapRect := DrawItemBitmap(Index, ARect);
    IF odSelected in State THEN BEGIN
      IF FBoldStyle THEN BEGIN
        IF fsBold IN Font.Style
          THEN Font.Style := Font.Style - [fsBold]
          ELSE Font.Style := Font.Style + [fsBold];
      END
      ELSE BEGIN
        Brush.Color := clHighlight;
        Font.Color := clHighlightText;
        FillRect(Rect(ARect.Left, ARect.Top, BitmapRect.Left, ARect.Bottom));
        FillRect(Rect(BitmapRect.Right, ARect.Top, ARect.Right, ARect.Bottom));
      END;
      {+++ Hint aktualisieren +++}
      UpdateHint(Index);
    END;
  END;

{************** Tabulatoren ermitteln *******************}
  GetMem(TabArray, Header.Sections.Count * sizeof(integer));
  try
    TabMerker := 0;
    for i := 0 to Header.Sections.Count - 1 do begin
      P := TabArray;
      Inc(P, I{*sizeof(Integer)});
      TabMerker := TabMerker + Header.Sections.Items[I].Width;
      P^ := TabMerker;
    end;

{************** Zeichnen *******************}
    with Canvas do begin
      Brush.Style := bsClear;
      IF Header.Visible AND (Header.Sections.Count > 0)
        THEN TabbedTextOut(Handle, ARect.Left + 2, ARect.Top, StrPCopy(ch, Items.Strings[Index]),
             Length(Items[Index]), Header.Sections.Count, TabArray^, 0)
        ELSE TextOut(ARect.Left + 2, ARect.Top, Items.Strings[Index]);
      Brush.Style := bsSolid;

{************** TabStops zeichnen? *******************}
      IF Header.Visible AND FLineByTabStops THEN
      FOR I := 0 TO Header.Sections.Count-1 DO BEGIN
        P := TabArray;
        Inc(P, I{*sizeof(Integer)});
        MoveTo(P^-3, ARect.Top);
        LineTo(P^-3, ARect.Bottom);
      end;

    end;
  finally
    IF Header.Sections.Count > 0 THEN FreeMem(TabArray, Header.Sections.Count * sizeof(integer));
  end;
end;

procedure TPieComboBox.SetItemProperties;
VAR
  EditDC: HDC;
begin
  //nur wenn Style = csSimple oder csDropDown
  //nur wenn gltiger ItemIndex
  IF NOT(Style IN [csSimple, csDropDown]) THEN exit;
  IF ItemIndex < 0 THEN exit;

  EditDC := GetDC(EditHandle);
  TRY
    {$R-}
    IF FAnyFonts THEN
      SendMessage(EditHandle, wm_SetFont, Items.Fonts[ItemIndex].Handle, 1);
    IF FAnyColors THEN SelectObject(EditDC, CreateSolidBrush(Items.Colors[ItemIndex]));
  FINALLY
    ReleaseDC(EditHandle, EditDC);
  END;

end;

procedure TPieComboBox.SetAnyColors(Value: Boolean);
begin
  IF Value <> FAnyColors THEN BEGIN
    FAnyColors := Value;
    Repaint;
  END;
end;

procedure TPieComboBox.SetAnyFonts(Value: Boolean);
VAR
  Index, AHeight: Integer;
begin
  IF Value <> FAnyFonts THEN BEGIN
    FAnyFonts := Value;
    FOR Index:=0 TO Items.Count-1 DO BEGIN
      MeasureItem(Index, AHeight);
      Perform(CB_SETITEMHEIGHT, Index, AHeight);
    END;
    Repaint;
  END;
end;

procedure TPieComboBox.SetBitmapStyle(AStyle: TBitmapStyle);
begin
  if FBitmapStyle <> AStyle then begin
    FBitmapStyle := AStyle;
    Perform(CB_SETCURSEL, Perform(CB_GETCURSEL, 0, 0), 0);
    Repaint;
  end;
end;

procedure TPieComboBox.SetBoldStyle(AStyle: Boolean);
begin
  if FBoldStyle <> AStyle then begin
    FBoldStyle := AStyle;
    Perform(CB_SETCURSEL, Perform(CB_GETCURSEL, 0, 0), 0);
  end;
end;

procedure TPieComboBox.SetItemHeightStyle(AStyle: TItemHeightStyle);
VAR
  Index, AHeight: Integer;
begin
  if FItemHeightStyle <> AStyle then begin
    FItemHeightStyle := AStyle;
    FOR Index:=0 TO Items.Count-1 DO BEGIN
      MeasureItem(Index, AHeight);
      Perform(CB_SETITEMHEIGHT, Index, AHeight);
    END;
    Repaint;
  end;
end;

procedure TPieComboBox.SetLineByTabStops(Value: Boolean);
begin
  if FLineByTabStops <> Value then begin
    FLineByTabStops := Value;
    Invalidate;
  end;
end;

function TPieComboBox.GetHCVisible: Boolean;
begin
  Result := Header.Visible;
end;

procedure TPieComboBox.SetHCVisible(Value: Boolean);
begin
  IF Value = Header.Visible THEN exit;
  Header.Visible := Value;
  IF Value
    THEN Header.SetBounds(Left, Top-Header.Height, Width, Header.Height)
    ELSE Header.SetBounds(-10, 0, 0, Header.Height);
  Invalidate;
end;

function TPieComboBox.GetHCS: THeaderSections;
begin
  Result := Header.Sections;
end;

procedure TPieComboBox.SetHCS(Value: THeaderSections);
begin
  Header.Sections := Value;
end;

function TPieComboBox.GetHCImages: TCustomImageList;
begin
  Result := Header.Images;
end;

procedure TPieComboBox.SetHCImages(Value: TCustomImageList);
begin
  Header.Images := Value;
end;

function TPieComboBox.GetHCStyle: THeaderStyle;
begin
  Result := Header.Style;
end;

procedure TPieComboBox.SetHCStyle(Value: THeaderStyle);
begin
  Header.Style := Value;
end;

function TPieComboBox.GetHCFont: TFont;
begin
  Result := Header.Font;
end;

procedure TPieComboBox.SetHCFont(Value: TFont);
begin
  Header.Font := Value;
end;

function TPieComboBox.GetOnSectionClick: TSectionNotifyEvent;
begin
  Result := Header.OnSectionClick;
end;

procedure TPieComboBox.SetOnSectionClick(Value: TSectionNotifyEvent);
begin
  Header.OnSectionClick := Value;
end;

procedure TPieComboBox.HeaderSectionResize(HeaderControl: THeaderControl; Section: THeaderSection);
begin
  Invalidate;
end;

function TPieComboBox.ItemAtPos(Pos: TPoint; Existing: Boolean): Integer;
var
  Count: Integer;
  ItemRect: TRect;
  P, H: Integer;
begin
  if PtInRect(ClientRect, Pos) then
  begin
    Result := 0;
    Count := FItems.Count;
    IF Style = csSimple THEN P := 0 ELSE P := Height;
    while Result < Count do begin
      H := Perform(CB_GETITEMHEIGHT, Result, 0);
      ItemRect := Rect(0, P, Width, P+H);
      if PtInRect(ItemRect, Pos) then Exit;
      P := P + H;
      Inc(Result);
    end;
    if not Existing then Exit;
  end;
  Result := -1;
end;

procedure TPieComboBox.UpdateHint(Index: Integer);
VAR
  MausP: TPoint;
  Idx: Integer;
BEGIN
  Idx := Index;
  IF Idx < 0 THEN BEGIN
    GetCursorPos(MausP);
    MausP := ScreenToClient(MausP);
    Idx := ItematPos(MausP, True);
  END;
  IF Idx > -1 THEN BEGIN
    IF HelpContext <> Items.HelpContexts[Idx] THEN
      HelpContext := Items.HelpContexts[Idx];
    IF Hint <> Items.Hints[Idx] THEN BEGIN
      Hint := Items.Hints[Idx];
    END;
  END;
  Perform(CM_HINTSHOW, 0, 0);
  {Mauszeiger auf Eltern simulieren --> damit wird der Hint wieder aktualisiert}
  GetCursorPos(MausP);
  SendMessage(Parent.Handle, wm_MouseMove, 0, 0);
  SendMessage(Parent.Handle, wm_MouseMove, 0, MausP.X + (MausP.Y SHL 16));
END;
{******************************************************}
{******************************************************}
{******************************************************}
procedure TPieComboBoxItemsfenster.Prop_zeigen;
begin
  Zeile := ComboBox.ItemIndex;
  Zeilenlabel2.Caption := Str_Number(Zeile+1);
  Gesamtzeilenlabel2.Caption := Str_Number(ComboBox.Items.Count);

  IF Zeile > -1 THEN BEGIN
    TRY
      IF NOT(ComboBox.Items.Bitmaps[Zeile].Empty)
        THEN BitmapVorschau.Picture.Assign(ComboBox.Items.Bitmaps[Zeile])
        ELSE BitmapVorschau.Picture.Assign(NIL);
      ColorShape.Brush.Color := ComboBox.Items.Colors[Zeile];
      Edit.Color := ComboBox.Items.Colors[Zeile];
      FontBtn.Font := ComboBox.Items.Fonts[Zeile];
      Edit.Font := ComboBox.Items.Fonts[Zeile];
      Edit.Font.Height := -11;
      HelpUpDown.Position := ComboBox.Items.HelpContexts[Zeile];
      HelpEdit.Caption := IntToStr(ComboBox.Items.HelpContexts[Zeile]);
    EXCEPT
      ON EAccessViolation DO;
    END;
  END;
end;

procedure TPieComboBoxItemsfenster.ComboBoxClick(Sender: TObject);
begin
  Edit.Text := ComboBox.Items.Strings[ComboBox.ItemIndex];
  Prop_zeigen;
  Edit.SetFocus;
end;

procedure TPieComboBoxItemsfenster.ComboBoxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  IF (Button = mbLeft) THEN WITH ComboBox DO BEGIN
    IF (ItematPos(Point(X,Y), TRUE) >= 0) AND
       (ItematPos(Point(X,Y), TRUE) < Items.Count) THEN BEGIN
      Zieh_Item := ItematPos(Point(X,Y), TRUE);
    END;
  END;
end;

procedure TPieComboBoxItemsfenster.ComboBoxDragDrop(Sender, Source: TObject; X, Y: Integer);
VAR
  Dropposition: Integer;
  Alt, Neu: Integer;
begin
  Dropposition := ComboBox.ItemAtPos(Point(X,Y), TRUE);
  IF (Source = ComboBox) AND (Zieh_Item <> Dropposition) AND
     (DropPosition >= 0) AND (DropPosition < ComboBox.Items.Count) THEN BEGIN

    Alt := Zieh_Item;
    Neu := Dropposition;
    IF Alt < Neu THEN inc(Neu);
    ComboBox.Items.InsertItem(Neu, ComboBox.Items.Strings[Alt]);

    Alt := Zieh_Item;
    Neu := Dropposition;
    IF Alt < Neu THEN inc(Neu) ELSE inc(Alt);
    ComboBox.Items.PieComboBoxEintraege[Neu] := ComboBox.Items.PieComboBoxEintraege[Alt];

    Alt := Zieh_Item;
    Neu := Dropposition;
    IF Alt > Neu THEN inc(Alt);
    ComboBox.Items.Delete(Alt);
  END;
end;

procedure TPieComboBoxItemsfenster.InsertBtnClick(Sender: TObject);
CONST
  CRLF = #$0D+#$0A;
VAR
  S: string;
  I: Integer;
begin
  S := Edit.Text;
  IF (length(S) > 1) AND (copy(S, length(S)-2, 2) = CRLF) THEN
    S := copy(S, length(S)-2, 2);
  I := ComboBox.ItemIndex;
  IF I<0 THEN I:=0;
  ComboBox.Items.Insert(I, S);
  Prop_zeigen;
end;

procedure TPieComboBoxItemsfenster.DeleteBtnClick(Sender: TObject);
begin
  ComboBox.Items.Delete(ComboBox.ItemIndex);
  Prop_zeigen;
end;

procedure TPieComboBoxItemsfenster.ChangeBtnClick(Sender: TObject);
CONST
  CRLF = #$0D+#$0A;
VAR
  S: string;
begin
  S := Edit.Text;
  IF (length(S) > 1) AND (copy(S, length(S)-2, 2) = CRLF) THEN
    S := copy(S, length(S)-2, 2);
  ComboBox.Items.Strings[ComboBox.ItemIndex] := S;
  Prop_zeigen;
end;

procedure TPieComboBoxItemsfenster.ColorBtnClick(Sender: TObject);
begin
  ColorDialog.Color := ComboBox.Items.Colors[Zeile];
  IF ColorDialog.Execute THEN BEGIN
    ComboBox.Items.Colors[Zeile] := ColorDialog.Color;
    ComboBox.Repaint;
    Prop_zeigen;
  END;
end;

procedure TPieComboBoxItemsfenster.FormShow(Sender: TObject);
begin
  ComboBox.Align := alNone;
  ComboBox.Header.Top := ComboBox.Top;
  ComboBox.Top := ComboBox.Top + ComboBox.Header.Height;
  ComboBox.Height := ComboBox.Height - ComboBox.Header.Height;
  CASE ComboBox.ItemHeightStyle OF
  ihcFix        : IHStyle.ItemIndex := 0;
  ihcFont       : IHStyle.ItemIndex := 1;
  ihcBitmap     : IHStyle.ItemIndex := 2;
  ihcFontBitmap : IHStyle.ItemIndex := 3;
  END;
  CASE ComboBox.BitmapStyle OF
  bscLeft       : BStyle.ItemIndex := 0;
  bscRight      : BStyle.ItemIndex := 1;
  bscCenter     : BStyle.ItemIndex := 2;
  bscStretch    : BStyle.ItemIndex := 3;
  END;
  CASE ComboBox.BackGroundStyle OF
  bgscNone      : BGStyle.ItemIndex := 0;
  bgscCenter    : BGStyle.ItemIndex := 1;
  bgscStretch   : BGStyle.ItemIndex := 2;
  bgscPattern   : BGStyle.ItemIndex := 3;
  END;
  UT.Checked := ComboBox.HeaderControl;
  UTClick(UT);
  LBTS.Checked := ComboBox.LineByTabStops;
  HSGroup.ItemIndex := Integer(ComboBox.HeaderStyle);
  AnyColors.Checked := ComboBox.AnyColors;
  AnyFonts.Checked := ComboBox.AnyFonts;
  BoldStyle.Checked := ComboBox.BoldStyle;
  Edit.Clear;
  ComboBox.ItemIndex := ComboBox.Items.Count-1;
  IF ComboBox.ItemIndex > -1 THEN BEGIN
    Edit.Text := ComboBox.Items.Strings[ComboBox.ItemIndex];
    Prop_zeigen;
  END;
end;

procedure TPieComboBoxItemsfenster.EditEnter(Sender: TObject);
begin
  Edit.SelectAll;
end;

procedure TPieComboBoxItemsfenster.EditChange(Sender: TObject);
begin
  Prop_zeigen;
end;

procedure TPieComboBoxItemsfenster.FontBtnClick(Sender: TObject);
begin
  FontDialog.Font := ComboBox.Items.Fonts[Zeile];
  IF FontDialog.Execute THEN BEGIN
    ComboBox.Items.Fonts[Zeile] := FontDialog.Font;
    ComboBox.Repaint;
    Prop_zeigen;
  END;
end;

procedure TPieComboBoxItemsfenster.HintBtnClick(Sender: TObject);
begin
  HintDialog.Zeilentext := ComboBox.Items.Hints[Zeile];
  IF HintDialog.Execute THEN BEGIN
    ComboBox.Items.Hints[Zeile] := HintDialog.Zeilentext;
  END;
end;

procedure TPieComboBoxItemsfenster.LoeschBtnClick(Sender: TObject);
begin
  ComboBox.Clear;
end;

procedure TPieComboBoxItemsfenster.UTClick(Sender: TObject);
begin
  ComboBox.HeaderControl := UT.Checked;
  HSGroup.Enabled := UT.Checked;
  LBTS.Enabled := UT.Checked;
  HFButton.Enabled := UT.Checked;
end;

procedure TPieComboBoxItemsfenster.AnyColorsClick(Sender: TObject);
begin
  ComboBox.AnyColors := AnyColors.Checked;
end;

procedure TPieComboBoxItemsfenster.AnyFontsClick(Sender: TObject);
begin
  ComboBox.AnyFonts := AnyFonts.Checked;
end;

procedure TPieComboBoxItemsfenster.FormCreate(Sender: TObject);
begin
  ComboBox := TPieComboBox.Create(Self);
  ComboBox.Parent := EditPanel;
  ComboBox.Header.Parent := EditPanel;
  ComboBox.Top := 89;
  ComboBox.Left := 4;
  ComboBox.Style := csSimple;
  ComboBox.Height := 230;
  ComboBox.Align := alClient;
  ComboBox.OnClick := ComboBoxClick;
  ComboBox.OnMouseDown := ComboBoxMouseDown;
  ComboBox.OnDragDrop := ComboBoxDragDrop;
  HintDialog := TPieZeilenDialog.Create(Self);
  WITH HintDialog DO BEGIN
    PasswordChar := #0;
    Labeltext := 'Hint';
    Ueberschrift := 'Hint fr diesen Item eingeben';
    Oktext := '&Ok';
    Abbrechentext := '&Abbrechen';
    KommazuPunkt := False;
    Left := 384;
    Top := 56;
  end;
end;

procedure TPieComboBoxItemsfenster.FormDestroy(Sender: TObject);
begin
  HintDialog.Free;
  ComboBox.Free;
end;

procedure TPieComboBoxItemsfenster.BitmapBtnClick(Sender: TObject);
VAR
  B: TBitmap;
begin
  IF OpenPicDialog.Execute THEN BEGIN
    B := TBitmap.Create;
    TRY
      B.LoadFromFile(OpenPicDialog.FileName);
      ComboBox.Items.Bitmaps[Zeile] := B;
    FINALLY
      B.Destroy;
    END;
    ComboBox.Repaint;
    Prop_zeigen;
  END;
end;

procedure TPieComboBoxItemsfenster.IHStyleClick(Sender: TObject);
begin
  CASE IHStyle.ItemIndex OF
  0: ComboBox.ItemHeightStyle := ihcFix;
  1: ComboBox.ItemHeightStyle := ihcFont;
  2: ComboBox.ItemHeightStyle := ihcBitmap;
  3: ComboBox.ItemHeightStyle := ihcFontBitmap;
  END;
end;

procedure TPieComboBoxItemsfenster.BStyleClick(Sender: TObject);
begin
  CASE BStyle.ItemIndex OF
  0: ComboBox.BitmapStyle := bscLeft;
  1: ComboBox.BitmapStyle := bscRight;
  2: ComboBox.BitmapStyle := bscCenter;
  3: ComboBox.BitmapStyle := bscStretch;
  END;
end;

procedure TPieComboBoxItemsfenster.BoldStyleClick(Sender: TObject);
begin
  ComboBox.BoldStyle := BoldStyle.Checked;
end;

procedure TPieComboBoxItemsfenster.LBTSClick(Sender: TObject);
begin
  ComboBox.LineByTabStops := LBTS.Checked;
end;

procedure TPieComboBoxItemsfenster.HSGroupClick(Sender: TObject);
begin
  ComboBox.HeaderStyle := THeaderStyle(HSGroup.ItemIndex);
end;

procedure TPieComboBoxItemsfenster.HFButtonClick(Sender: TObject);
begin
  FontDialog.Font := ComboBox.HeaderFont;
  IF FontDialog.Execute THEN
    ComboBox.HeaderFont := FontDialog.Font;
end;

procedure TPieComboBoxItemsfenster.HelpUpDownChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  ComboBox.Items.HelpContexts[Zeile] := HelpUpDown.Position;
  HelpEdit.Caption := IntToStr(HelpUpDown.Position);
end;

procedure TPieComboBoxItemsfenster.BGButtonClick(Sender: TObject);
begin
  IF OpenPicDialog.Execute THEN
    ComboBox.BackGround.LoadFromFile(OpenPicDialog.FileName);
end;

procedure TPieComboBoxItemsfenster.BGStyleClick(Sender: TObject);
begin
  CASE BGStyle.ItemIndex OF
  0: ComboBox.BackGroundStyle := bgscNone;
  1: ComboBox.BackGroundStyle := bgscCenter;
  2: ComboBox.BackGroundStyle := bgscStretch;
  3: ComboBox.BackGroundStyle := bgscPattern;
  END;
end;

end.
