unit Viewcomb;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Menus, Dialogs, StdCtrls, DB, DBTables;

type
  TDBViewCombo = class;

  TViewComboLink = class(TDataLink)
  private
   FDBViewCombo : TDBViewCombo;
   FDataFieldName : string;
   FDataField : TField;
   FWasEdited : boolean;
   FReading : boolean;
   procedure SetDataFieldName(const Value : string);
   procedure SetDataField;

  protected
   procedure ActiveChanged;override;
   procedure DataSetScrolled(Distance : integer);override;
   procedure DataSetChanged;override;
   procedure EditingChanged;override;
{   procedure LayoutChanged;override;}
  public
   constructor Create(ADBViewCombo : TDBViewCombo);
   property DataField: TField read FDataField;
   property DataFieldName: string read FDataFieldName write SetDataFieldName;
  end;

  TDBViewCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FDataLink : TViewComboLink;
    FOldIndex : integer;
    FChanging : boolean;
    function GetDataField: string; { returns the name of the data field }
    function GetDataSource: TDataSource; { returns reference to the data source }
    procedure SetDataField(const Value: string); { assigns name of data field }
    procedure SetDataSource(Value: TDataSource); { assigns new data source }
    function GetItemIndex : integer;
    procedure SetItemIndex(Value : integer);

  protected
    { Protected declarations }
    procedure Scroll(Distance : Integer);
    procedure ReadList;   {read and fill the list of items (when dataset activeting)}
    procedure ClearList;  {clear list}
    procedure Change;override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure CreateWnd;override;
    procedure Notification(Component : TComponent;Operation : TOperation);override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
    property ItemIndex : integer read GetItemIndex write SetItemIndex;
  published
    { Published declarations }
    property Align;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property Style; {Must be published before Items}
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ItemHeight;
{    property Items;}
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
{    property Sorted;}
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
  end;

procedure Register;

implementation

function MemComp(Buf1,Buf2 : PChar; Size : Word) : boolean;
var
 i : integer;
begin
 i := 0;
 while i < Size do
 begin
  if Buf1[i] <> Buf2[i]
  then begin Result := false;exit; end;
  inc(i);
 end;
 Result := true;
end;

constructor TViewComboLink.Create(ADBViewCombo : TDBViewCombo);
begin
 inherited Create;
 FDBViewCombo := ADBViewCombo;
 FDataFieldName := '';
 FDataField := nil;
 FWasEdited := false;
 FReading := false;
end;

procedure TViewComboLink.SetDataField;
begin
 if DataSource = nil then exit;
 if DataSource.DataSet = nil then exit;
 if not DataSource.DataSet.Active then exit;
 if FDataFieldName = '' then exit;
 FDataField := DataSource.DataSet.FieldByName(FDataFieldName);
 if Active
 then FDBViewCombo.ReadList;
end;

procedure TViewComboLink.SetDataFieldName(const Value: string);
begin
 if FDataFieldName <> Value
 then begin
  FDataFieldName := Value;
  SetDataField;
 end;
end;

procedure TViewComboLink.ActiveChanged;
begin
 if Active
 then begin
  if FDataField = nil
  then begin
   SetDataField;
//   FDBViewCombo.ReadList;
  end;
 end
 else begin
  FDataField := nil;
  FDBViewCombo.ClearList;
 end
end;

procedure TViewComboLink.DataSetChanged;
begin
  if FReading then exit;
  FReading := true;
  try
    if not (DataSource.DataSet.State in dsEditModes) then
    begin
      FDBViewCombo.ClearList;
      FDBViewCombo.ReadList;
    end;
  finally
    FReading := false;
  end;  
end;

procedure TViewComboLink.DataSetScrolled(Distance : integer);
begin
  if Distance = 0 then exit;
  FDBViewCombo.Scroll(Distance);
end;

procedure TViewComboLink.EditingChanged;
begin
 if Editing
 then FWasEdited := true;
end;

{---------------------------------------------------------------}
constructor TDBViewCombo.Create(AOwner: TComponent);
begin
 inherited Create(AOwner); { always call the inherited constructor first }
 FDataLink := TViewComboLink.Create(Self); { construct the data-link object }
 Style := csDropDownList;
 FOldIndex := -1;
 FChanging := false;
end;

destructor TDBViewCombo.Destroy;
begin
  FDataLink.Free; { always destroy owned objects first... }
  inherited Destroy; { ...then call inherited destructor }
end;

procedure TDBViewCombo.Notification(Component : TComponent;Operation : TOperation);
begin
 inherited Notification(Component,Operation);
 if (FDataLink <> nil) and (Component = DataSource) and (Operation = opRemove)
 then DataSource := nil;
end;

function TDBViewCombo.GetItemIndex : integer;
begin
 Result := inherited ItemIndex;
end;

procedure TDBViewCombo.SetItemIndex(Value : integer);
begin
 inherited ItemIndex := Value;
 Change;
 FOldIndex := Value;
end;

function TDBViewCombo.GetDataField: string;
begin
 Result := FDataLink.DataFieldName;
end;

function TDBViewCombo.GetDataSource: TDataSource;
begin
 Result := FDataLink.DataSource;
end;

procedure TDBViewCombo.SetDataField(const Value: string);
begin
 FDataLink.DataFieldName := Value;
end;

procedure TDBViewCombo.SetDataSource(Value: TDataSource);
begin
 FDataLink.DataSource := Value;
end;

procedure TDBViewCombo.Change;
begin
 if not FChanging and (FDataLink.DataSource <> nil) and
    (FDataLink.DataSource.DataSet <> nil) and (FDataLink.DataField <> nil)
 then begin
  FChanging := true;
  FDataLink.DataSource.DataSet.MoveBy(ItemIndex-FOldIndex);
  FOldIndex := ItemIndex;
  FChanging := false;
 end;
 inherited Change;
end;

procedure TDBViewCombo.Scroll(Distance : integer);
begin
 if FChanging then exit;  
 FChanging := true;
 ItemIndex := ItemIndex + Distance;
 FChanging := false;
end;

procedure TDBViewCombo.ReadList;
var
 Bmrk : TBookMark;
 CurRecBuffer,RecBuffer : PChar;
 n,Index : integer;
begin
 if FDataLink.DataField = nil then exit;
 if FDataLink.DataSource = nil then exit;
 if FDataLink.DataSource.DataSet = nil then exit;
 if not FDataLink.DataSource.DataSet.Active then exit;
 Items.Clear;
 with FDataLink.DataSource.DataSet do
 begin
  CurRecBuffer := StrAlloc(RecordSize);
  RecBuffer := StrAlloc(RecordSize);
  GetCurrentRecord(CurRecBuffer);
  Bmrk := GetBookMark;
  DisableControls;
  First;n := 0;
  while not EOF do
  begin
   Items.Add(FieldByName(FDatalink.FDataFieldName).AsString);
   GetCurrentRecord(RecBuffer);
   if MemComp(RecBuffer,CurRecBuffer,RecordSize) then Index := n;
   Next;inc(n);
  end;
  GotoBookmark(Bmrk);
  FreeBookmark(Bmrk);
  EnableControls;
  FChanging := true;
  ItemIndex := Index;
  FOldIndex := Index;
  FChanging := false;
  StrDispose(RecBuffer);
  StrDispose(CurRecBuffer);
 end;

end;

procedure TDBViewCombo.CreateWnd;
begin
 inherited CreateWnd;
 ItemIndex := FOldIndex;
end;

procedure TDBViewCombo.ClearList;
begin
 Items.Clear;
end;

procedure TDBViewCombo.KeyUp(var Key: Word; Shift: TShiftState);
begin
 {}
end;

procedure TDBViewCombo.Refresh;
begin
 ReadList;
end;

procedure Register;
begin
  RegisterComponents('Data Controls', [TDBViewCombo]);
end;

end.
