unit gfglIDEExplore;

{

Explore Items Delphi IDE enhancement. copyright Greg Lorriman.
greg@lorriman.com http://www.lorriman.com

This tool is given over to the Delphi programming comuunity, commercial and non-commercial,
with a single requirement :

If you modify the code and wish to re-publish this utility with your enhancements
then you must inform me (email address above) and add a comment at the top of the source
indicating your changes. You must also preserve this message. Furthermore you should
explicitly direct support email to yourself. You must also supply a proper
disclaimer (one is provided with the readme.txt file).

Otherwise this utility is completely free.

Source notes :

This utility is very rough. I have simply taken a poor example of using the toolsapi
from the demos directory and modified it for my own purposes. The most important method
is : "Execute" along with the helper function "CurrentProjectGroup".

Work that needs doing :

Keyboard shortcuts for the three menu items.
Relocation of the menu items to somewhere other than the "Help" menu.
Re-write to use better method for doing experts (ie. not IOTAWizardServices.AddWizard)

}


interface

uses
   SysUtils, ToolsApi, windows, classes, menus, dialogs, extCtrls;

type
   EIDEExploreItem = class(Exception);

   TExploreType = (etFile, etProject, etProjectGroup);
   TIDEExploreItem = class(TNotifierObject, IOTAWizard, IOTAMenuWizard)
   private
      FIndex: Integer;
      FExploreType: TExploreType;
      FTimer: TTimer;
      procedure SetIndex(const Value: Integer);
      procedure SetExploreType(const Value: TExploreType);
      procedure OnTimer(sender: tobject);
   public
      constructor create;
      destructor destroy; override;

      function GetIDString: string;
      function GetName: string;
      function GetState: TWizardState;
      procedure Execute;
      function GetMenuText: string;
    //for IOTAKeyBindingServices.addKeyBinding
      procedure BindKey(const Context: IOTAKeyContext; KeyCode: TShortcut;
         var BindingResult: TKeyBindingResult);

      property Index: Integer read FIndex write SetIndex;
      property ExploreType: TExploreType read FExploreType write SetExploreType;
   end;

implementation

//uses DbugIntf;

const
   etStrings: array[etFile..etProjectGroup] of string = ('File', 'Project', 'Project Group');

var
   FileService: TIDEExploreItem;
   ProjectService: TIDEExploreItem;
   ProjectGroupService: TIDEExploreItem;

resourcestring
   sActionError = 'Error creating ExploreItemService wizard';

function CurrentProjectGroup: IOTAProjectGroup; forward;

{ TIDEExploreItem }

procedure TIDEExploreItem.Execute;
var
   filename: string;
   group: IOTAProjectGroup;
   services: IOTAModuleServices;
   info: IOTAModule40;
begin
   if FExploreType = etFile then begin
      services := BorlandIDEServices as IOTAModuleServices;
      if services.CurrentModule <> nil then begin
         info := services.CurrentModule as IOTAModule40;
         if info <> nil then
            filename := info.getFileName;
      end;
   end;

   if FExploreType in [etProject, etProjectGroup] then begin
      group := currentProjectGroup;
      if group <> nil then begin
         if FExploreType = etProjectGroup then
            filename := (group as IOTAModule40).filename
         else if FExploreType = etProject then
            filename := (group.activeProject as IOTAModule40).filename;
      end;
   end;

   if filename <> '' then
      winexec(pchar('explorer ' + extractFilePAth(filename)), sw_show);
end;

function TIDEExploreItem.GetIDString: string;
begin
   Result := 'Lorrisoft.DelphiIDEEnhancement.ExploreItemService.1.' + etStrings[FExploreType]; { do not localize }
end;

function TIDEExploreItem.GetMenuText: string;
begin
   Result := 'Explore ' + etStrings[FExploreType];
end;

function TIDEExploreItem.GetName: string;
begin
   Result := 'Lorrisoft.DelphiIDEEnhancement.ExploreItemService.' + etStrings[FExploreType]; { do not localize }
end;

function TIDEExploreItem.GetState: TWizardState;
begin
   Result := [wsEnabled];
end;

function CurrentProjectGroup: IOTAProjectGroup;
var
   Services: IOTAModuleServices;
   I: Integer;
begin
   Services := BorlandIDEServices as IOTAModuleServices;
   for I := 0 to Services.ModuleCount - 1 do
      if Services.Modules[I].QueryInterface(IOTAProjectGroup, Result) = S_OK then
         Exit;
   Result := nil;
end;

procedure InitActionServices;
   procedure addService(ExploreItemService: TIDEExploreItem);
   begin
      if (BorlandIDEServices <> nil) then begin
         ExploreItemService.Index := (BorlandIDEServices as IOTAWizardServices).AddWizard(ExploreItemService as IOTAWizard);
         if ExploreItemService.Index < 0 then
            raise EIDEExploreItem.Create(sActionError);
      end;
   end;
begin
   ProjectGroupService := TIDEExploreItem.Create;
   ProjectGroupService.ExploreType := etProjectGroup;
   addService(ProjectGroupService);

   ProjectService := TIDEExploreItem.Create;
   ProjectService.ExploreType := etProject;
   addService(ProjectService);

   FileService := TIDEExploreItem.Create;
   FileService.ExploreType := etFile;
   addService(FileService);
//    FileService.Ftimer.enabled:=true;
end;

procedure DoneActionServices;
begin
   if (BorlandIDEServices <> nil) then begin
      (BorlandIDEServices as IOTAWizardServices).RemoveWizard(FileService.Index);
      (BorlandIDEServices as IOTAWizardServices).RemoveWizard(ProjectService.Index);
      (BorlandIDEServices as IOTAWizardServices).RemoveWizard(ProjectGroupService.Index);
   end;
end;

procedure TIDEExploreItem.SetIndex(const Value: Integer);
begin
   FIndex := Value;
end;

procedure TIDEExploreItem.BindKey(const Context: IOTAKeyContext;
   KeyCode: TShortcut; var BindingResult: TKeyBindingResult);
begin
   bindingResult := krHandled;
//   if keyCode=ShortCut(ord('#'),[ssctrl,ssShift]) then

   execute;

end;

constructor TIDEExploreItem.create;
begin
   inherited;
   FTimer := TTimer.Create(nil);
   Ftimer.enabled := false;
   FTimer.interval := 4000;
   Ftimer.OnTimer := onTimer;
end;

destructor TIDEExploreItem.destroy;
begin
   FTimer.free;
   inherited;
end;

procedure TIDEExploreItem.OnTimer(sender: tobject);
var
   menu: TMenuItem;
   i: integer;
begin
   try
      if FExploreType <> etFile then EXIT;
//find menuitem name

  // find Delphi's main menu
  // find File
//      SendSeparator;
      menu := (BorlandIDEServices as INTAServices).MainMenu.Items.Find('Help');
      if menu <> nil then begin
//         for i := 0 to menu.count - 1 do
//            senddebug(menu.items[i].name);
         menu := menu.find('Explore File');
         if menu <> nil then begin
//            senddebug('menuFound');
            if not (BorlandIDEServices as IOTAKeyBindingServices).addKeyBinding([ShortCut(ord('G'), [ssctrl])], bindKey, nil, 7, '', menu.name) then
               showmessage('not bound');

         end;
      end;
   except
   end;
   Ftimer.enabled := false;
end;

{ TIDEExploreItem }

procedure TIDEExploreItem.SetExploreType(const Value: TExploreType);
begin
   FExploreType := Value;
end;

initialization
   InitActionServices;
finalization
   DoneActionServices;
end.

