unit u_SockUtil;
{** Based on Sockutil.c in
 ** "Network Windows Programming" by Alok K. Sinha
 **}
interface
uses
  Windows,
  Winsock;

function WinSockInitialize : integer;
function GetHostAddress(const hostname : string) : u_long;
function GetLocalHostAddress : u_long;
function GetPortForService(const serviceName, protocol : string) : u_short;
function BindSocketToAddr(iBindType : integer ;
                          soc : TSocket;
                          servicename : string;
                          const protocolname : string;
                          fClient : BOOL) : integer;
function CreateSocket(iBindType  : integer; type_ : integer) : TSocket;
procedure DestroySocket(socket : TSocket);
function ConnectToServer(iBindType : integer; soc : TSocket ;
                         HostName : string ; ServiceName : string ; ProtocolName : string) : integer;


function GetHostAddressIPStr(const hostname : string) : string;
function SocketErrorMsg(Error: integer) : string;

const
  NETBIOS_NAME_LENGTH = 16;
  DEFAULT_PORT_ADDR = 1;
  FIXED_PORT_LOCAL_ADDR = 2;
  GET_PORT_AND_LOCAL_ADDR = 3;
  BIND_NETBIOS = 4;
  BIND_IPX = 5;
  GET_PORT_AND_REMOTE_ADDR = 6;
  FIXED_PORT = 80;
  CLIENT_QUEUE_SIZE = 5;
  READ_WRITE_LOOP = 1;
  SHUTDOWN = 'ShutDown';
type
  TSockAddr_nb = packed record
    snb_family : short;
    snb_type   : u_short;
    snb_name   : array[0..NETBIOS_NAME_LENGTH-1] of char;
  end;
  PSockAddr_nb = ^TSockAddr_nb;

  TSockAddr_ipx = packed record
    sa_family : short;
    sa_netnum : array[0..4-1] of char;
    sa_nodenum : array[0..6-1] of char;
    sa_socket : u_short;
  end;
  PSockAddr_ipx = ^TSockAddr_ipx;

implementation

uses
  SysUtils;

const
  NETBIOS_UNIQUE_NAME  = 0;
  NSPROTO_IPX = 0;
  NSPROTO_SPX = 1256;
  NSPROTO_SPXII = 1257;

  defIPXAddress : TSockAddr_ipx = ( sa_family : AF_IPX              ;
                                    sa_netnum : (#0,#0,#0,#0)       ;
                                    sa_nodenum: (#0,#0,#0,#0,#0,#0) ;
                                    sa_socket : $4040
                                  );
procedure SET_NETBIOS_SOCKADDR(snb : PSockAddr_nb ; type_ : word ; name : PCHAR ; port : char);
var
  i : integer;
begin
  snb^.snb_family := AF_NETBIOS;
  snb^.snb_type := type_;
  for i := 0 to NETBIOS_NAME_LENGTH - 1 do
  begin
    snb^.snb_name := ' ';
  end;
  i := 0;
  while (name[i] <> #0) and (i < NETBIOS_NAME_LENGTH-1) do
  begin
    snb^.snb_name[i] := name[i];
  end;
  snb^.snb_name[NETBIOS_NAME_LENGTH - 1] := port;
end;

function WinSockInitialize : integer;
var
  wVersionRequired : WORD;
  wsaData : TwsaData;
begin
  wVersionRequired := MAKEWORD(1,1);
  result := WSAStartup(wVersionRequired, wsaData);
end;

function GetHostAddress(const hostname : string) : u_long;
var
  pHostAddr : PHostEnt;
type
  T = ^u_long;
begin
  pHostAddr := gethostbyname(PCHAR(hostname));
  if (pHostAddr = nil) then
  begin
    result := 0;
  end
  else
  begin
    result := T(pHostAddr^.h_addr^)^;
  end;
end;

function GetLocalHostAddress : u_long;
var
  szHostName : array[0..MAX_PATH] of char;
begin
  if (gethostname(szHostName, SizeOf(szHostName)) <> SOCKET_ERROR) then
    result := GetHostAddress(szHostName)
  else
    result := 0;
end;

function GetPortForService(const serviceName, protocol : string) : u_short;
var
  pServAddr : PServent;
begin
  pServAddr := getservbyname(PCHAR(servicename), PCHAR(protocol));
  if (pServAddr <> nil) then
    result := pServAddr^.s_port
  else
    result := u_short(-1);
end;

function BindSocketToAddr(iBindType : integer ;
                          soc : TSocket;
                          servicename : string;
                          const protocolname : string;
                          fClient : BOOL) : integer;
var
  LocalAddr : TSockAddrIn;
  NetBiosAddr : TSockAddr_nb;
  IpxAddr     : TSockAddr_ipx;
  err         : integer;
  ulHostAddress : u_long;
  usPortNo  : u_short;
  bReuse : BOOL;
  szNbName : array[0..NETBIOS_NAME_LENGTH - 1] of char;
  tmp : pchar;
begin
  bReuse := true;
  if (fClient) then
  begin
    if ((iBindType <> BIND_IPX) and (iBindType <> BIND_NETBIOS)) then
      iBindType := DEFAULT_PORT_ADDR;
  end;
  FillChar(LocalAddr, SizeOf(LocalAddr), 00);
//  err := SOCKET_ERROR;
  case (iBindType) of
    DEFAULT_PORT_ADDR :
    begin
      LocalAddr.sin_family := AF_INET;
      LocalAddr.sin_addr.S_addr := htonl(INADDR_ANY);
      LocalAddr.sin_port := 0;
    end;
    FIXED_PORT_LOCAL_ADDR :
    begin
      ulHostAddress := GetLocalHostAddress;
      if (ulHostAddress = 0) then
      begin
        result := -1;
        exit;
      end;
      LocalAddr.sin_family := AF_INET;
      LocalAddr.sin_addr.s_addr := ulHostAddress;
      LocalAddr.sin_port := htons(FIXED_PORT);
    end;
    GET_PORT_AND_LOCAL_ADDR :
    begin
      ulHostAddress := GetLocalHostAddress;
      if (ulHostAddress = 0) then
      begin
        result := -1;
        exit;
      end
      else
        usPortNo := GetPortForService(ServiceName, ProtocolName);
      if (usPortNo = 65535) then
        usPortNo := htons(StrToIntDef(ServiceName, -1));
      LocalAddr.sin_family := AF_INET;
      LocalAddr.sin_addr.s_addr := ulHostAddress;
      LocalAddr.sin_port := usPortNo;
    end;
    BIND_NETBIOS :
    begin
      usPortNo := GetPortForService(serviceName, ProtocolName);
      if (usPortNo = 65535) then
        usPortNo := htons(StrToIntDef(ServiceName, -1));
      if (fClient) then
      begin
        serviceName := 'WSockClient';
      end;
      FillChar(szNbName, SizeOf(szNbName), 0);
      Move(serviceName, szNbName, Length(ServiceName));
      SET_NETBIOS_SOCKADDR(@NetBiosAddr, NETBIOS_UNIQUE_NAME, szNbName, CHAR(usPortNo));
    end;
    BIND_IPX:
    begin
      move(defIPXAddress, IpxAddr, SizeOf(TSOCKADDR_IPX));
      tmp := PCHAR(ServiceName);
      move(tmp, ipxAddr.sa_nodenum, 6);
      tmp := PCHAR(ProtocolName);
      move(tmp, ipxAddr.sa_netnum, 4);
      if (fClient) then
        ipxAddr.sa_socket := 0;
      setsockopt(soc, SOL_SOCKET, SO_REUSEADDR, PCHAR(bReuse), 4);
    end;
  end;

  if (iBindType = BIND_NETBIOS) then
    err := bind(soc, TSockAddrIn(pSockAddrIn(@NetBiosAddr)^), SizeOf(NetBiosAddr))
  else if (iBindType = BIND_IPX) then
    err := bind(soc, TSockAddrIn(pSockAddrIn(@IpxAddr)^), SizeOf(IpxAddr))
  else
    err := bind(soc, TSockAddrIn(pSockAddrIn(@LocalAddr)^), SizeOf(LocalAddr));
  result := err;
end;

function CreateSocket(iBindType  : integer; type_ : integer) : TSocket;
var
  soc : TSocket;
begin
  case (iBindType) of
    BIND_IPX :
    begin
      if (type_ = SOCK_STREAM) then
        soc := Socket(AF_IPX, type_, NSPROTO_SPX)
      else
        soc := Socket(AF_IPX, type_, NSPROTO_IPX);
    end;
    BIND_NETBIOS :
    begin
      if (type_ = SOCK_STREAM) then
        soc := Socket(AF_IPX, type_, SOCK_SEQPACKET)
      else
        soc := Socket(AF_IPX, type_, SOCK_DGRAM);
    end;
    else
    begin
      soc := Socket(AF_INET, type_, 0);
    end;
  end;
  result := soc;
end;


function ConnectToServer(iBindType : integer; soc : TSocket ;
                         HostName : string ; ServiceName : string ; ProtocolName : string) : integer;
var
  RemAddr : TSockAddrIn;
  NetBiosAddr : TSockAddr_NB;
  IpxAddr : TSockAddr_ipx;
  err : integer;
  ulHostAddress : u_long;
  usPortNo : u_short;
begin
  FillChar(RemAddr, 0, SizeOf(TSockAddrIn));
  case (iBindType) of
    DEFAULT_PORT_ADDR :
    begin
      result := SOCKET_ERROR;
      exit;
    end;
    FIXED_PORT_LOCAL_ADDR :
    begin
      ulHostAddress := GetLocalHostAddress;
      if (ulHostAddress = 0) then
      begin
        result := SOCKET_ERROR;
        exit;
      end;
      RemAddr.sin_family := AF_INET;
      RemAddr.sin_addr.S_addr := UlHostAddress;
      RemAddr.sin_port := htons(FIXED_PORT);
    end;
    GET_PORT_AND_LOCAL_ADDR :
    begin
      ulHostAddress := GetLocalHostAddress;
      if (ulHostAddress = 0) then
      begin
        result := SOCKET_ERROR;
        exit;
      end;
      usPortNo := (GetPortForService(ServiceName, ProtocolName));
      if (usPortNo = 65535) then
        usPortNo := htons(StrToIntDef(ServiceName, -1));
      RemAddr.sin_family := AF_INET;
      RemAddr.sin_addr.s_addr := ulHostAddress;
      RemAddr.sin_port := usPortNo;
    end;
    GET_PORT_AND_REMOTE_ADDR :
    begin
      ulHostAddress := GetHostAddress(HostName);
      if (ulHostAddress = 0) then
      begin
        result := SOCKET_ERROR;
        exit;
      end;
      usPortNo := (GetPortForService(ServiceName, ProtocolName));
      if (usPortNo = 65535) then
        usPortNo := htons(StrToIntDef(ServiceName, -1));
      RemAddr.sin_family := AF_INET;
      RemAddr.sin_addr.s_addr := ulHostAddress;
      RemAddr.sin_port := usPortNo;
    end;
    BIND_IPX :
    begin
      Move(defIPXaddress, IpxAddr, SizeOf(TSockAddr_IPX));
    end;
  end;
  if (iBindType = BIND_NETBIOS) then
    err := connect(soc, TSockAddrIn(pSockAddrIn(@NetBiosAddr)^), SizeOf(NetBiosAddr))
  else if (iBindType = BIND_IPX) then
    err := connect(soc, TSockAddrIn(pSockAddrIn(@IpxAddr)^), SizeOf(IpxAddr))
  else
    err := Connect(soc, RemAddr, SizeOf(RemAddr));
  result := Err;
end;

//WMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWM
//WMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWM
//WMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWM

function GetHostAddressIPStr(const hostname : string) : string;
var
  adr : u_long;
  in_ : TInAddr;
begin
  adr := GetHostAddress(hostname);
  in_.S_addr := adr;
  result := inet_ntoa(in_);
end;

function SocketErrorMsg(Error: integer) : string;
begin
  case Error of
    WSAEINTR:
      SocketErrorMsg := 'Interrupted system call';
    WSAEBADF:
      SocketErrorMsg := 'Bad file number';
    WSAEACCES:
      SocketErrorMsg := 'Permission denied';
    WSAEFAULT:
      SocketErrorMsg := 'Bad address';
    WSAEINVAL:
      SocketErrorMsg := 'Invalid argument';
    WSAEMFILE:
      SocketErrorMsg := 'Too many open files';
    WSAEWOULDBLOCK:
      SocketErrorMsg := 'Operation would block';
    WSAEINPROGRESS:
      SocketErrorMsg := 'Operation now in progress';
    WSAEALREADY:
      SocketErrorMsg := 'Operation already in progress';
    WSAENOTSOCK:
      SocketErrorMsg := 'Socket operation on non-socket';
    WSAEDESTADDRREQ:
      SocketErrorMsg := 'Destination address required';
    WSAEMSGSIZE:
      SocketErrorMsg := 'Message too long';
    WSAEPROTOTYPE:
      SocketErrorMsg := 'Protocol wrong type for socket';
    WSAENOPROTOOPT:
      SocketErrorMsg := 'Protocol not available';
    WSAEPROTONOSUPPORT:
      SocketErrorMsg := 'Protocol not supported';
    WSAESOCKTNOSUPPORT:
      SocketErrorMsg := 'Socket type not supported';
    WSAEOPNOTSUPP:
      SocketErrorMsg := 'Operation not supported on socket';
    WSAEPFNOSUPPORT:
      SocketErrorMsg := 'Protocol family not supported';
    WSAEAFNOSUPPORT:
      SocketErrorMsg := 'Address family not supported by protocol family';
    WSAEADDRINUSE:
      SocketErrorMsg := 'Address already in use';
    WSAEADDRNOTAVAIL:
      SocketErrorMsg := 'Can''t assign requested address';
    WSAENETDOWN:
      SocketErrorMsg := 'Network is down';
    WSAENETUNREACH:
      SocketErrorMsg := 'Network is unreachable';
    WSAENETRESET:
      SocketErrorMsg := 'Network dropped connection on reset';
    WSAECONNABORTED:
      SocketErrorMsg := 'Software caused connection abort';
    WSAECONNRESET:
      SocketErrorMsg := 'Connection reset by peer';
    WSAENOBUFS:
      SocketErrorMsg := 'No buffer space available';
    WSAEISCONN:
      SocketErrorMsg := 'Socket is already connected';
    WSAENOTCONN:
      SocketErrorMsg := 'Socket is not connected';
    WSAESHUTDOWN:
      SocketErrorMsg := 'Can''t send after socket shutdown';
    WSAETOOMANYREFS:
      SocketErrorMsg := 'Too many references: can''t splice';
    WSAETIMEDOUT:
      SocketErrorMsg := 'Connection timed out';
    WSAECONNREFUSED:
      SocketErrorMsg := 'Connection refused';
    WSAELOOP:
      SocketErrorMsg := 'Too many levels of symbolic links';
    WSAENAMETOOLONG:
      SocketErrorMsg := 'File name too long';
    WSAEHOSTDOWN:
      SocketErrorMsg := 'Host is down';
    WSAEHOSTUNREACH:
      SocketErrorMsg := 'No route to host';
    WSAENOTEMPTY:
      SocketErrorMsg := 'Directory not empty';
    WSAEPROCLIM:
      SocketErrorMsg := 'Too many processes';
    WSAEUSERS:
      SocketErrorMsg := 'Too many users';
    WSAEDQUOT:
      SocketErrorMsg := 'Disc quota exceeded';
    WSAESTALE:
      SocketErrorMsg := 'Stale NFS file handle';
    WSAEREMOTE:
      SocketErrorMsg := 'Too many levels of remote in path';
    WSASYSNOTREADY:
      SocketErrorMsg := 'Network sub-system is unusable';
    WSAVERNOTSUPPORTED:
      SocketErrorMsg := 'WinSock DLL cannot support this application';
    WSANOTINITIALISED:
      SocketErrorMsg := 'WinSock not initialized';
    WSAHOST_NOT_FOUND:
      SocketErrorMsg := 'Host not found';
    WSATRY_AGAIN:
      SocketErrorMsg := 'Non-authoritative host not found';
    WSANO_RECOVERY:
      SocketErrorMsg := 'Non-recoverable error';
    WSANO_DATA:
      SocketErrorMsg := 'No Data';
  else
    SocketErrorMsg := 'Unknown WinSock error';
  end;
end;

procedure DestroySocket(socket : TSocket);
begin
  Winsock.Shutdown(socket,1);
  CloseSocket(Socket);
end;

end.


