unit U_Http;
{** U_Http.pas
 ** 05/02/2001 by Jerome Forestier
 ** Unit to get raw HTML result. Use low level winsock
 ** api (in U_SockUtil.pas), so it can be used in console app.
 ** Usefull for checking exploit.
 **}

interface

Uses
  Windows,
  Winsock,
  U_SockUtil;

const
  RCV_BUFSIZE = 512;            // Size of the incomming data buffer
  CRLF = #$0D#$0A;              //
  DEFAULT_TIMEOUT_SECOND = 10;   //
  // Error Code
  OK = 0;
  ERR_BIND      = -1;
  ERR_CONNECT   = -2;
  ERR_SEND      = -3;
  ERR_TIMEOUT   = -4;
  ERR_SELECT    = -5;
  ERR_RECV      = -6;
  ERR_PROXY_CONNECT = -7;
type
  THttpRequest = packed record
    Site : string;          // in : site address (www.yahoo.com)
    Path : string;          // in : site path (/index.html)
    Port : string;          // in : site port number (80)
    UseProxy : boolean;     // in : use proxy for getting data
    ProxyAddress : String;  // in : proxy address (proxy.toto.com)
    ProxyPort : string;     // in : proxy port (3128)
    Timeout : word ;        // in : tiemout for getting data in seconds
    Result : string;        // out : raw html result
    ResultLen : integer;    // out : number of bytes returned
    ResultCode : integer;   // out : error code (if any)
    ResultTime : longint;   // out : time, in ms, to get the data
    HttpCode : word;        // in / out if 1, the http header will be parsed and the error code (404) returned in
                            //          if 0, no check for error code
  end;

function HttpGet(const site, path, port : string ; var res : string) : integer;
// The simpliest function to retreive html source of a page.
// Raw html is in res variable.
// The function return 0 if ok or an error code (ERR_xxx)

Function HttpGetEx(var Request : THttpRequest) : integer;
// Extended function to retreive html source of a page
// Request parameters is in/out. Check out THttpRequest type
// The function return the number of bytes retreived (> 0) if ok
// or an error code (ERR_xxx)

function HttpGetLow(var Request : THttpRequest ; header : string) : integer;
// Low HTTP level . Request.Site is ignored. header must contains GET and other
// header statements.

implementation
uses
  sysutils;
procedure InitSocket;
var
  wsData : TWsAData;
begin
  WSAStartup($0101, wsData);
end;

procedure DeInitSocket;
begin
  WSACleanup;
end;

procedure CheckHttpHeader(buf :PCHAR ; len : integer ; var errorcode : word);
// This is experimental !
// We enter in this proc only for the first incomming buffer,
// it _should_ contain the http header with the error code
// eg : HHTP/1.0 404 Not Founf
var
  i : integer;
  code : integer;
  scode : pchar;
begin
  if (ErrorCode <> 1) then
    exit;
  i := 0;
  // We search for the first space
  while (buf[i] <> ' ') and (buf[i] <> #0) and (i <= len) do
  begin
    inc(i);
  end;
  if (buf[i] = ' ') then
  begin
    // char after the space are the error code
    scode := @buf[i+1];
    code := StrToIntDef(Copy(scode,1,3), 0);
    errorcode := code;
  end
  else
    ErrorCode := 0;
end;

Function HttpGetEx(var Request : THttpRequest) : integer;
label
  EndOfProc;
var
  Socket     : TSocket;
  iBindType  : Integer;
  err        : integer;
  ToSend     : string;
  SndBuf     : PCHAR;
  RcvBuf     : array[0..RCV_BUFSIZE] of char;
  FD         : TFDSet;
  TimeOut    : TTimeVal;
  len        : integer;
  Time       : longint;
begin
  Time := GetTickCount;
  result := -1;
  Request.Result := '';
  Request.ResultCode := 0;
  Request.ResultLen := 0;
  TimeOut.tv_sec  := Request.Timeout;
  TimeOut.tv_usec := 0;
  FD_ZERO(FD);
  iBindType := GET_PORT_AND_REMOTE_ADDR;
  socket := CreateSocket(iBindType, SOCK_STREAM);
  err := BindSocketToAddr(iBindType,
                          socket,
                          request.Port,
                          'TCP',
                          true);
  if (err = SOCKET_ERROR) then
  begin
    Request.ResultCode := ERR_BIND;
    Goto EndOfProc;
  end;

  if (Request.UseProxy) then
  begin
    err := ConnectToServer(iBindType, Socket,
                           Request.ProxyAddress, Request.ProxyPort, 'TCP');
    if (err = SOCKET_ERROR) then
      Request.ResultCode := ERR_PROXY_CONNECT;
  end
  else
  begin
    err := ConnectToServer(iBindType, Socket,
                           Request.Site, Request.Port, 'TCP');
    if (err = SOCKET_ERROR) then
      Request.ResultCode := ERR_CONNECT;
  end;
  if (err = SOCKET_ERROR) then
    Goto EndOfProc;

  if (Request.UseProxy) then
  begin
    ToSend := Format('GET http://%s:%s%s HTTP/1.0'+CRLF, [Request.Site, Request.Port, Request.Path]);
  end
  else
  begin
    ToSend := Format('GET %s HTTP/1.0'+CRLF, [Request.Path]);
  end;
  ToSend := ToSend+'Accept: */*'+CRLF;
  ToSend := ToSend+'Accept-Language: en'+CRLF;
  ToSend := ToSend+'Accept-Encoding: gzip'+CRLF;
  ToSend := ToSend+'User-Agent: Mozilla/4.7 [fr] (AmstradOS; I)'+CRLF;
  ToSend := ToSend+'Host: '+Request.Site+':'+Request.port+CRLF;
  if (Request.UseProxy) then
    ToSend := ToSend + 'Proxy-Connection: Keep-Alive'+CRLF
  else
    ToSend := ToSend + 'Connection: Keep-Alive'+CRLF;
  ToSend := ToSend + CRLF+CRLF;
  len := Length(ToSend) - 1;
  SndBuf := PCHAR(ToSend);
  FD_SET(socket, FD);
  err := send(Socket, SndBuf^, len, 0);
  if (err = SOCKET_ERROR) then
  begin
    Request.ResultCode := ERR_SEND;
    goto EndOfProc;
  end;
  repeat
    err := Select(FD_SETSIZE, @FD, nil, nil, @TimeOut);
    if (err = 0) then
    begin
      Request.ResultCode := ERR_TIMEOUT;
      break;
    end
    else if (err = SOCKET_ERROR) then
    begin
      Request.ResultCode := ERR_SELECT;
      break;
    end;
    if (FD_ISSET(socket, FD)) then
    begin
      err := recv(Socket, RcvBuf, RCV_BUFSIZE, 0);
      if (err = SOCKET_ERROR) then
      begin
        Request.ResultCode := ERR_RECV;
        break;
      end
      else if (err <> 0) then
      begin
        rcvBuf[err] := #0;
        if (Request.HttpCode = 1) then
          CheckHttpHeader(RcvBuf, RCV_BUFSIZE, Request.HttpCode);
        Request.Result := Request.Result + rcvBuf;
        Request.ResultLen := Request.ResultLen + err;
      end;
    end;
  until (err = 0);
EndOfProc :
  DestroySocket(socket);
  Request.ResultTime := GetTickCount - Time;
  result := Request.ResultLen;
end;

function HttpGet(const site, path, port : string ; var res : string) : integer;
var
  socket : TSocket;
  iBindType : integer;
  err : integer;
  p : PCHAR;
  RcvBuf : array[0..RCV_BUFSIZE] of char;
  FD : TFDSet;
  TimeOut : TTimeVal;
  len : integer;
begin
  FillChar(rcvBuf, RCV_BUFSIZE, 0);
  TimeOut.tv_sec  := DEFAULT_TIMEOUT_SECOND;
  TimeOut.tv_usec := 0;
  res := '';
  FD_ZERO(FD);
  iBindType := GET_PORT_AND_REMOTE_ADDR;
  socket := CreateSocket(iBindType, SOCK_STREAM);
  err := BindSocketToAddr(iBindType,
                          socket,
                          port,
                          'TCP',
                          true);
  if (err = SOCKET_ERROR) then
  begin
    result := ERR_BIND;
    exit;
  end;
  err := ConnectToServer(iBindType, Socket,
                         site, port, 'TCP');
  if (err = SOCKET_ERROR) then
  begin
    result := ERR_CONNECT;
    exit;
  end;
  p := PCHAR(Format('GET %s HTTP/1.0'+CRLF+CRLF+CRLF, [path]));
  len := length(p) - 1;
  FD_SET(socket, FD);
  err := send(Socket, p^, len, 0);
  if (err = SOCKET_ERROR) then
  begin
    result := ERR_SEND;
    exit;
  end;
  repeat
    err := Select(FD_SETSIZE, @FD, nil, nil, @TimeOut);
    if (err = 0) then
    begin
      result := ERR_TIMEOUT;
      exit;
    end
    else if (err = SOCKET_ERROR) then
    begin
      result := ERR_SELECT;
      exit;
    end;
    if (FD_ISSET(socket, FD)) then
    begin
      err := recv(Socket, rcvBuf, RCV_BUFSIZE, 0);
      if (err = SOCKET_ERROR) then
      begin
        result := ERR_RECV;
        break;
      end
      else if (err <> 0) then
      begin
        rcvBuf[err] := #0;
        res := res + rcvBuf;
      end;
    end;
  until (err = 0);
  DestroySocket(socket);
  result := Ok;
end;

function HttpGetLow(var Request : THttpRequest ; header : string) : integer;
begin
end;

initialization
  initSocket;
finalization
  DeInitSocket;
end.
